# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.views.decorators.http import require_GET
from django.template import RequestContext

from django.shortcuts import (
    render_to_response,
    get_object_or_404,
    redirect,
)

from django_tables2 import RequestConfig

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)

from common.views import index

from common.models import Bug

from bootspeed.models import (
    Image,
    Machine,
    ImageResult,
    ImageBuild,
    JobImage,
)

from bootspeed.tables import MachineTable

CHART_WIDTH_DAYS = 60


def _get_bootspeed_arches():
    arches = []
    for image in Image.objects.values(
        'arch'
    ).distinct().exclude(publish=False).order_by('-arch'):
        arches.append(image['arch'])

    return arches


def _get_bootspeed_releases():
    releases = []
    for image in Image.objects.values('release').distinct().exclude(
        publish=False
    ).order_by('-release'):
        releases.append(image['release'])

    return releases


@require_GET
def overview(request, release=None):
    images = Image.objects.all()
    releases = _get_bootspeed_releases()

    if release is not None:
        images = images.filter(release=release)

    data = {
        'images': images,
        'releases': releases,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(overview),
    }

    return render_to_response(
        'bootspeed/overview.html',
        data,
        RequestContext(request)
    )


@BreadCrumb("Bootspeed testing", parent=index, needs=['arch', ])
@require_GET
def arch_overview(request, arch=None):
    if arch is None:
        return redirect('arch/amd64/')

    result = ImageResult.objects.filter(image__arch=arch).distinct(
        'machine__name'
    )
    arches = _get_bootspeed_arches()
    data = {
        'result': result,
        'arch': arch,
        'arches': arches,
        'active': arch,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            arch_overview,
            arch=arch,
        ),
    }
    return render_to_response(
        'bootspeed/arch_overview.html',
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Aggregated data",
    parent=arch_overview,
    needs=['machine_id', 'arch'],
)
@require_GET
def machine_data(request, machine_id, arch):
    machine = get_object_or_404(Machine, id=machine_id)

    data = {
        'name': machine.name,
        'id': machine.id,
        'arch': arch,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            machine_data,
            machine_id=machine_id,
            machine=machine.name,
            arch=arch,
        ),
    }
    return render_to_response(
        "bootspeed/machine_data.html",
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Raw data",
    parent=machine_data,
    needs=['machine_id', 'arch'],
)
@require_GET
def machine_raw_data(request, machine_id, arch):
    machine = get_object_or_404(Machine, id=machine_id)

    qs = ImageResult.objects.filter(
        machine__id=machine_id,
        image__arch=arch,
    ).prefetch_related(
        'image',
        'machine',
    )

    data = []
    for q in qs:
        data.append(dict(
            machine=q.machine,
            image=q.image,
            image__build_number=q.image.build_number,
            image__arch=q.image.arch,
            image__release=q.image.release,
            image__variant=q.image.variant,
            kernel=q.kernel,
            kernel_init=q.kernel_init,
            plumbing=q.plumbing,
            xorg=q.xorg,
            desktop=q.desktop,
            boot=q.boot,
            files=q.log_links,
        ))

    table = MachineTable(data)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = {
        'name': machine.name,
        'id': machine.id,
        'arch': arch,
        'table': table,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            machine_raw_data,
            machine_id=machine_id,
            machine=machine.name,
            arch=arch,
        ),
    }
    return render_to_response("bootspeed/machine_raw_data.html", data,
                              RequestContext(request))


@BreadCrumb(
    "Bugs by image",
    parent=arch_overview,
)
@require_GET
def image_bugs_overview(request):
    images = JobImage.objects.annotate(
        imagebuild_count=models.Count(
            'imagebuild'
        )
    ).filter(imagebuild_count__gt=0).order_by(
        '-build_number',
        'arch'
    ).exclude(build_number="UNKNOWN")

    unknown_images = JobImage.objects.annotate(
        imagebuild_count=models.Count(
            'imagebuild')
    ).filter(imagebuild_count__gt=0).order_by(
        '-build_number',
        'arch'
    ).filter(build_number="UNKNOWN")

    data = {
        'images': images,
        'unknown_images': unknown_images,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_bugs_overview,
            arch='amd64',
        ),
    }

    return render_to_response(
        "bootspeed/image_bugs_overview.html",
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "{image}",
    parent=image_bugs_overview,
    needs=['image_id']
)
@require_GET
def image_bugs_detail(request, image_id):
    image_builds = ImageBuild.objects.filter(job_image=image_id)
    image = get_object_or_404(JobImage, id=image_id)

    data = {
        'image_builds': image_builds,
        'image': image,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_bugs_detail,
            image_id=image_id,
            arch='amd64',
            image=image.build_number,
        ),
    }

    return render_to_response(
        "bootspeed/image_bugs_detail.html",
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Bugs",
    parent=arch_overview,
)
@require_GET
def bug_overview(request):
    bugs = Bug.objects.annotate(
        build_count=models.Count('build_bugs')
    ).filter(build_count__gt=0).order_by('-build_count')

    bug_numbers = []
    for bug in bugs:
        if bug.bug_no not in bug_numbers:
            bug_numbers.append(bug.bug_no)

    data = {
        'bugs': bugs,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            bug_overview,
            arch='amd64',
        ),
    }

    return render_to_response(
        "bootspeed/bug_overview.html",
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Builds affected",
    parent=bug_overview,
    needs=['bug_id'],
)
@require_GET
def bug_detail(request, bug_id):
    bug = get_object_or_404(Bug, id=bug_id)

    data = {
        'builds': bug.build_bugs.all(),
        'bug': bug,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            bug_detail,
            arch='amd64',
            bug_id=bug_id,
        ),
    }

    return render_to_response(
        "bootspeed/bug_detail.html",
        data,
        RequestContext(request)
    )
