# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import importlib
import logging

from django.template.loader import render_to_string

from plugins import PLUGIN_APPS


class Extension(object):
    """ A definition of a QA Dashboard Plugin.

    A plugin app should include one sub-class of this class named
    'extension' in <app>/dashboard.py
    """

    LINKS = []

    def contribute_to_kpis(self, kpi_mgr):
        """ The KPIs exposed by the plugin. """
        return []

    def pull_data(self):
        """ The entry point to be called to retrieve the data used by the
        extension.
        """

        pass

    def name(self):
        """ The name of the plugin. """
        raise NotImplementedError()


class KPI(object):
    """ A KPI entry. """
    IMPORTANCE_CRITICAL = 0
    IMPORTANCE_HIGH = 10
    IMPORTANCE_MEDIUM = 20
    IMPORTANCE_LOW = 30

    @classmethod
    def name(self):
        """ The displayable name for a given KPI. """
        raise NotImplementedError()

    def importance(self):
        """ Gives us a way to order KPI's in the view how we'd like. """
        raise NotImplementedError()

    def value(self):
        """ The value of the KPI used for combining KPIs. """

        pass

    def render(self):
        """ Creates the HTML view of the KPI. """
        raise NotImplementedError()


class KPIManager(object):
    """ A manager class for plugin's KPI entries. """
    def __init__(self):
        self._kpis = {}

    def add_kpi(self, kpi):
        """ Gets or creates an instance of the KPI class. """

        # XXX: store kpis based on name to support multiple extensions
        # contributing to the same kpi.
        kpi = kpi()
        self._kpis.setdefault(
            kpi.importance(),
            {}
        ).setdefault(
            kpi.name(),
            []
        ).append(kpi)

    def render(self):
        """ Render the KPIs. """
        snippets = []
        for importance in sorted(self._kpis.keys()):
            for name in sorted(self._kpis[importance].keys()):
                kpi = self._kpis[importance][name]
                snippet = "<div class='grid_7'>"
                snippet += "<div class='box box-padded kpi'>\n"
                snippet += "<h3>{}</h3>\n".format(name)
                snippet += "<div>\n"
                for k in kpi:
                    snippet += '{}\n'.format(k.render())
                snippet += "</div>\n"
                snippet += "</div></div>\n"
                snippets.append(snippet)
                # TODO inset snippet into array based on importance

        return '\n'.join(snippets)


class ExtensionManager(object):
    """ Class for managing extensions. """

    def _get_extensions(self):
        for app in PLUGIN_APPS:
            mod = importlib.import_module("{}.dashboard".format(app))
            self.extensions.append(mod.extension)

    def __init__(self):
        """ Query all plugin apps for extensions. """

        self.extensions = []
        self._get_extensions()

    def get_extension_data(self):
        """ Retrieve data on behalf of any extensions with a pull_data()
        method.
        """

        for ext in self.extensions:
            # don't allow an extension to halt our progress
            try:
                ext.pull_data()
            except Exception as e:
                logging.warn("{}: {}".format(ext.name(), e))

    def kpi_render(self):
        """ Main entry point for KPI rendering.

        Creates a KPIManager and processes all the KPIs for all extensions.

        """

        kpi_manager = KPIManager()

        for ext in self.extensions:

            kpis = ext.contribute_to_kpis(kpi_manager)

            for kpi_cls in kpis:
                kpi_manager.add_kpi(kpi_cls)

        output = kpi_manager.render()

        return output

    def get_urls(self):
        """ Get plugin url configs. """

        urls = []
        for ext in self.extensions:

            if hasattr(ext, 'URL_PATTERNS'):
                urls += ext.URL_PATTERNS

        return urls

    def get_links(self, url):
        """ Get plugin navigation links. """

        links = ''
        link_index = {}

        for ext in self.extensions:

            for link in ext.LINKS:
                retval = render_to_string(
                    'nav_entry.html',
                    dict(
                        link=link,
                        url=url,
                    ),
                )
                link_index.setdefault(link['index'], '')
                link_index[link['index']] += retval

        for index in sorted(link_index.keys()):
            links += link_index[index]

        return links
