# QA Dashboard
# Copyright 2012-2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re

from django import template
from django.template.defaultfilters import stringfilter, floatformat

from common.models import Bug
from common.plugin_helper import ExtensionManager

from smokeng.config import get_internal_jenkins
from smokeng.models import SmokeResult

register = template.Library()

device_names = {
    'flo': 'Nexus 7 2013',
    'grouper': 'Nexus 7',
    'mako': 'Nexus 4',
    'maguro': 'Galaxy Nexus',
    'manta': 'Nexus 10',
}

product_names = {
    'flo': 'nexus7',
    'grouper': 'nexus7',
    'mako': 'nexus4',
    'maguro': 'Galaxy Nexus',
    'manta': 'nexus10',
}


def pass_rate_color(value):
    """Colouring for pass rates based on css classes"""
    if value == 1.0:
        return "green"
    elif value > 0.6:
        return "yellow"
    else:
        return "red"

register.filter('pass_rate_color', pass_rate_color)


def pass_fail_color(value):
    """ Coloring for pass/fail values.  """

    # True for failed, False for passed.
    if value:
        return "red"
    else:
        return "green"

register.filter('pass_fail_color', pass_fail_color)


def not_zero_color(value):
    """ Coloring for count values. Zero is bad """

    if value == 0:
        return "red"
    else:
        return "green"

register.filter('not_zero_color', not_zero_color)


def zero_color(value):
    """ Coloring for count values. Zero is good """

    if value == 0:
        return "green"
    else:
        return "red"

register.filter('zero_color', zero_color)


def bug_list(bugs, prefix=""):
    html = ""

    # handle a single bug and a list of bugs.
    if isinstance(bugs, Bug):
        bugs = [bugs]

    bug_numbers = set(bugs)

    html_fmt = '<a href="https://bugs.launchpad.net/bugs/{}/">{}LP:#{}</a>'

    for bug in bug_numbers:
        html = " ".join([
            html,
            html_fmt.format(bug.bug_no, prefix, bug.bug_no)
        ])

    return html

register.filter('bug_list', bug_list)


def binary_suffix(num):
    """ Assumes default units are K. """

    if num == "":
        return num

    LIMIT = 1024
    orig_num = num

    suffixes = ['K', 'M', 'G', 'T', 'P']
    index = 0

    while num >= LIMIT:
        index += 1
        num = float(num) / LIMIT

    if index >= len(suffixes):
        index = 0
        num = orig_num

    # format float values but avoid scientific notation.
    # inspired by: http://stackoverflow.com/a/2440786
    num = "{:.2f}".format(num).rstrip('0').rstrip('.')
    return "{}{}".format(num, suffixes[index])

register.filter('binary_suffix', binary_suffix)


def percent(numerator, denominator):
    if denominator == 0:
        return 0
    return float(numerator) / denominator * 100

register.filter('percent', percent)


@register.filter
def decimal_to_percent(value):
    if value is None:
        return None
    return floatformat(value * 100) + '%'


@register.filter
@stringfilter
def add_to_class(value, extra=""):

    if len(value) < 7:
        return value

    # value must be of the form 'class="BLAH"'
    prefix = value[:7]
    postfix = value[-1]

    if prefix == 'class="' and postfix == '"':
        retval = "{}{} {}{}".format(prefix, value[7:-1], extra, postfix)
        return retval

    return value


@register.filter
def display_name(device):
    try:
        clean = device.split('-')[0]
        return device + ' (' + device_names[clean] + ')'
    except KeyError:
        return device


@register.filter
def product_name(device):
    try:
        clean = device.split('-')[0]
        return product_names[clean]
    except KeyError:
        return device


@register.filter(name='getkey')
def getkey(value, arg):
    return value[arg]


# Template tags
@register.simple_tag(name='plugin_links')
def do_plugin_links(url):
    ''' Get plugin links for all plugins. '''
    em = ExtensionManager()
    links = em.get_links(url)

    return links


@register.filter
def internal_url(url):
    regex = re.compile(
        ur'^http(|s)://jenkins.qa.ubuntu.com/(job/)([^/]+)(/.*)',
    )
    m = regex.match(url)
    job = m.group(3)
    build = m.group(4)
    if 'touch' in job:
        jenkins = get_internal_jenkins(variant='touch')
    else:
        jenkins = get_internal_jenkins(variant='')
    jenkins_url = (jenkins + m.group(2) + job + build)
    return jenkins_url


@register.filter
def smokeresult_ran_at(smokeresult):
    status = smokeresult['status']
    if status == SmokeResult.COMPLETE:
        return smokeresult['ran_at']
    else:
        for s in SmokeResult.STATUS_CHOICES:
            if s[0] == status:
                return s[1]
    raise RuntimeError('Unexpected smokeresult status: %d' % status)
