# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase

from common.models import Bug
from common.templatetags.dashboard_extras import (
    binary_suffix,
    percent,
    bug_list,
    display_name,
    internal_url,
)


class TestDashboardExtras(TestCase):

    def test_binary_suffix(self):
        self.assertEqual(binary_suffix(1024), '1M')
        self.assertEqual(binary_suffix(1025), '1M')
        self.assertEqual(binary_suffix(1525), '1.49M')
        self.assertEqual(binary_suffix(1), '1K')
        self.assertEqual(binary_suffix(1023), '1023K')
        self.assertEqual(binary_suffix(4096), '4M')
        self.assertEqual(binary_suffix(1048576), '1G')
        self.assertEqual(binary_suffix(1073741824), '1T')
        self.assertEqual(binary_suffix(1099511627776), '1P')
        self.assertEqual(binary_suffix(1125899906842624), '1125899906842624K')

    def test_percent(self):
        self.assertEqual(percent(0, 2), 0)
        self.assertEqual(percent(0, 0), 0)
        self.assertEqual(percent(1, 3), 1.0 / 3 * 100)
        self.assertEqual(percent(1, 2), 50)
        self.assertEqual(percent(9, 9), 100)

    def _check_html(self, html, count):
        data = html.split('a href')

        self.assertEqual(count + 1, len(data))

    def test_bug_list_with_single_bug(self):
        bug = Bug.objects.create(bug_no=1)

        html = bug_list(bug)

        self._check_html(html, 1)

    def test_bug_list_with_prefix(self):
        bug = Bug.objects.create(bug_no=1)
        prefix = "BLAH"

        html = bug_list(bug, prefix)

        self.assertIn(prefix, html)

    def test_bug_list(self):
        bugs = []

        for i in range(3):
            bug, new_bug = Bug.objects.get_or_create(
                bug_no=i,
            )
            bugs.append(bug)

        html = bug_list(bugs)

        self._check_html(html, len(bugs))

    def test_display_name_with_suffix(self):
        device = 'mako-01'
        display = display_name(device)
        self.assertEquals(display, 'mako-01 (Nexus 4)')

    def test_display_name_without_suffix(self):
        device = 'mako'
        display = display_name(device)
        self.assertEquals(display, 'mako (Nexus 4)')

    def test_display_name_not_matching(self):
        device = 'nexus702'
        display = display_name(device)
        self.assertEquals(display, 'nexus702')

    def test_display_name_not_matching_with_suffix(self):
        device = 'nexus-702'
        display = display_name(device)
        self.assertEquals(display, 'nexus-702')

    def test_internal_url_touch(self):
        url = (
            'http://jenkins.qa.ubuntu.com/job/saucy-touch_ro-maguro-smoke-' +
            'ubuntu-terminal-app-autopilot/85/artifact/clientlogs/adb.' +
            'props/*view*/'
        )
        expected = (
            'http://q-jenkins.ubuntu-ci:8080/job/saucy-touch_ro-maguro-smoke-' +
            'ubuntu-terminal-app-autopilot/85/artifact/clientlogs/adb.' +
            'props/*view*/'
        )
        internal = internal_url(url)
        self.assertEquals(internal, expected)

    def test_internal_url_desktop(self):
        url = (
            'http://jenkins.qa.ubuntu.com/job/saucy-desktop-amd64-smoke-' +
            'encrypted_home/159/artifact/utah-15532-saucy-amd64_encrypted_' +
            'home.run_2013-10-10_10-10-43.yaml/*view*/'
        )
        expected = (
            'http://d-jenkins.ubuntu-ci:8080/job/saucy-desktop-amd64-smoke-' +
            'encrypted_home/159/artifact/utah-15532-saucy-amd64_encrypted_' +
            'home.run_2013-10-10_10-10-43.yaml/*view*/'
        )
        internal = internal_url(url)
        self.assertEquals(internal, expected)

    def test_internal_url_server(self):
        url = (
            'http://jenkins.qa.ubuntu.com/job/saucy-server-amd64-smoke-' +
            'postgresql-server/155/artifact/log/utah-15523.syslog.log/*view*/'
        )
        expected = (
            'http://d-jenkins.ubuntu-ci:8080/job/saucy-server-amd64-smoke-' +
            'postgresql-server/155/artifact/log/utah-15523.syslog.log/*view*/'
        )
        internal = internal_url(url)
        self.assertEquals(internal, expected)
