# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mock

from django.test.client import Client
from django.core.urlresolvers import reverse
from django.test import TestCase
from common.plugin_helper import (
    KPI,
    KPIManager,
    Extension,
    ExtensionManager,
)


class TestPlugins(TestCase):

    def setUp(self):
        self.c = Client()
        self.kpi_manager = KPIManager()
        self.ext_manager = ExtensionManager()

        class MyExtension(Extension):

            def name(self):
                return "extension"

        self.extension = MyExtension()
        self.assert_(self.extension)

        class MyKPI(KPI):
            @classmethod
            def name(cls):
                return "name"

            def importance(self):
                return KPI.IMPORTANCE_HIGH

            def render(self):
                return "render"

        self.kpi_cls = MyKPI

    def test_ext_sub_class_has_required_methods(self):

        class MyExtension(Extension):
            pass

        ext = MyExtension()
        self.assertRaises(NotImplementedError, ext.name)

    def test_ext_sub_class_has_required_defaults(self):

        self.assertEqual(self.extension.name(), "extension")
        self.assertEqual(
            self.extension.contribute_to_kpis(self.kpi_manager),
            [],
        )

        self.extension.pull_data()

    def test_kpi_sub_class_has_required_methods(self):

        class MyKPI(KPI):
            pass

        kpi = MyKPI()
        self.assertRaises(NotImplementedError, kpi.name)
        self.assertRaises(NotImplementedError, kpi.importance)
        self.assertRaises(NotImplementedError, kpi.render)

    def test_kpi_sub_class(self):

        kpi = self.kpi_cls()
        self.assertEqual(self.kpi_cls.name(), "name")
        self.assertEqual(kpi.importance(), KPI.IMPORTANCE_HIGH)
        self.assertEqual(kpi.render(), "render")

    def test_kpi_manager_add_kpi(self):

        self.kpi_manager.add_kpi(self.kpi_cls)
        self.assertEqual(len(self.kpi_manager._kpis), 1)

    def test_kpi_manager_render(self):

        kpi_manager = mock.MagicMock(KPIManager)
        kpi_manager._kpis = [self.kpi_cls]

        self.assertNotEqual(kpi_manager.render(), "")

    # These tests patch _get_extensions to avoid looking for extensions
    @mock.patch('common.plugin_helper.ExtensionManager._get_extensions')
    def test_ext_manager(self, mock_method):

        ext_manager = ExtensionManager()
        self.assert_(ext_manager)

        self.assertEqual(len(ext_manager.extensions), 0)

    @mock.patch('common.plugin_helper.ExtensionManager._get_extensions')
    def test_ext_manager_kpi_render_no_extensions(self, mock_method):

        ext_manager = ExtensionManager()
        self.assert_(ext_manager)

        self.assertEqual(ext_manager.kpi_render(), "")

    @mock.patch('common.plugin_helper.KPIManager.render')
    @mock.patch('common.plugin_helper.ExtensionManager._get_extensions')
    @mock.patch('common.plugin_helper.Extension.contribute_to_kpis')
    def test_ext_manager_kpi_render_with_extension(
        self,
        contrib,
        get_exts,
        render,
    ):

        ext_manager = ExtensionManager()
        self.assert_(ext_manager)

        contrib.return_value = [self.kpi_cls]
        ext_manager.extensions = [self.extension]

        output = "ext_manager_render"
        render.return_value = output

        self.assertEqual(ext_manager.kpi_render(), output)

    @mock.patch('common.plugin_helper.ExtensionManager._get_extensions')
    def test_ext_manager_get_ext_data_handles_exceptions(self, _get_ext):

        ext_manager = ExtensionManager()
        self.assert_(ext_manager)

        self.extension.pull_data = mock.Mock(side_effect=KeyError("error"))
        ext_manager.extensions = [self.extension]

        # shouldn't raise and exception
        ext_manager.get_extension_data()

    @mock.patch('common.plugin_helper.ExtensionManager._get_extensions')
    def test_ext_manager_get_ext_data(self, _get_ext):

        ext_manager = ExtensionManager()
        self.assert_(ext_manager)

        self.extension.pull_data = mock.Mock()
        ext_manager.extensions = [self.extension]

        ext_manager.get_extension_data()

        # ensre that our mocked pull_data() was called
        self.extension.pull_data.called_once_with()

    @mock.patch('common.plugin_helper.ExtensionManager._get_extensions')
    def test_index_url(self, get_ext):

        url = reverse('index', )
        response = self.c.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(url, 'kpi.html')
