# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import test as djangotest
from django.test.client import Client
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from smokeng.models import SmokeImage


class UserProfileTestCase(djangotest.TestCase):

    def setUp(self):
        self.c = Client()
        self.user = User.objects.create(
            username='testuser',
            first_name='Test',
            last_name='User',
        )
        self.user.set_password('password')
        self.user.save()

        self.image = SmokeImage.objects.create(
            release="saucy",
            arch="mako",
            variant="touch_mir",
            build_number="20130401",
        )
        self.assert_(self.image)

    def tearDown(self):
        self.c.logout()

    def login(self):
        logged_in = self.c.login(username='testuser', password='password')
        self.assertTrue(logged_in)

    def _get_page_url(self):
        url = reverse(
            'smokeng_overview',
            args=(
                self.image.release,
            ),
        )
        return url

    def test_not_logged_in_no_edit_profile_link(self):
        """
        Test that a user who is not logged in does not have a link to edit
        their profile
        """
        response = self.c.get(self._get_page_url())
        self.assertContains(response, '/edit/', 0)

    def test_logged_in_edit_profile_link(self):
        """
        Test that a user that is logged in has a link to edit their profile
        """
        self.login()
        response = self.c.get(self._get_page_url())
        self.assertContains(response, '/edit/', 1)

    def test_not_logged_in_edit_profile_url(self):
        """
        Test that a user who tries to edit their profile without being logged
        in is redirected
        """
        response = self.c.get(reverse('edit_profile'))
        self.assertEquals(response.status_code, 302)

    def test_edit_profile_url(self):
        """
        Test edit profile url
        """
        self.login()
        response = self.c.get(reverse('edit_profile'))
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'form.html')

    def test_user_save_profile(self):
        """
        Tests that a user can update their profile
        """
        self.login()
        response = self.c.get(reverse('edit_profile'))
        self.assertEqual(response.status_code, 200)

        # Define data to fill our the form
        use_private_url = True
        user = self.user
        csrf_token = '%s' % response.context['csrf_token']

        # Post the form
        post = self.c.post(
            reverse('edit_profile'),
            data={
                'csrfmiddlewaretoken': csrf_token,
                'use_private_url': use_private_url,
                'user': user,
            }
        )

        self.assertEqual(post.status_code, 302)
