# QA Dashboard

# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import json

from django.db import models
from django.views.decorators.http import require_GET
from django.shortcuts import (
    get_object_or_404,
)

from common.utils import (
    JSONResponse,
    MySerializer,
)

from eventstat.utils import (
    get_images,
    get_machines,
    get_machine_metrics,
)
from eventstat.models import (
    EventstatMachine,
    EventstatDetail,
    EventstatImage,
    EventstatUpgrade,
)


@require_GET
def images(request, arch=None):
    images = get_images(arch=arch)

    serializer = MySerializer(use_extra_fields=True)
    data = serializer.serialize(images)

    return JSONResponse(data)


@require_GET
def machines(request, arch=None):
    machines = get_machines(arch=arch)

    serializer = MySerializer(use_extra_fields=True)
    data = serializer.serialize(machines)

    return JSONResponse(data)


@require_GET
def machine_overview(request, machine_id, method, name=None):

    machine = get_object_or_404(EventstatMachine, id=machine_id)
    metrics = get_machine_metrics(
        machine=machine,
        name=name,
        method=method,
    )

    if method == "image":
        metrics = metrics.order_by(
            'image__rootfs_id',
            'image__build_id',
            'image__device_specific_id',
            'image__customization_id',
            'image__build_number',
        )
    else:
        metrics = metrics.order_by('upgrade__date')

    image_data = []
    for metric in metrics.iterator():
        image_info = dict(
            bottom=metric.minimum,
            min=metric.minimum,
            max=metric.maximum,
            avg=metric.average,
            std=metric.stddev,
        )
        if method == "image":
            image_info['build_number'] = metric.image.build_number
        else:
            image_info['build_number'] = metric.upgrade.date.strftime(
                "%Y-%m-%d",
            )

        image_data.append(image_info)

    bundle = dict(image_data=image_data)
    data = json.dumps(bundle)

    return JSONResponse(data)


@require_GET
def task_overview(request, machine_id, image_id, task):
    machine = get_object_or_404(EventstatMachine, id=machine_id)
    image = get_object_or_404(EventstatImage, id=image_id)
    tasks = EventstatDetail.objects.filter(
        publish=True,
        task=task,
        machine=machine,
        image__build_number__lte=image.build_number,
    ).values(
        'image__build_number',
    ).annotate(
        average=models.Avg('total'),
        minimum=models.Min('total'),
        maximum=models.Max('total'),
        stddev=models.StdDev('total'),
    ).order_by(
        'image__build_number',
    )
    last_release_avg = 0.0
    last_release_name = 'Unknown'
    bundle = {
        'last_release_avg': last_release_avg,
        'last_release_name': last_release_name,
    }
    task_data = []
    for t in tasks:
        task_data.append(
            dict(
                build_number=t['image__build_number'],
                bottom=t['minimum'],
                min=t['minimum'],
                max=t['maximum'],
                avg=t['average'],
                std=t['stddev'],
            ),
        )

    bundle['count'] = tasks.count()
    bundle['image_data'] = task_data
    data = json.dumps(bundle)

    return JSONResponse(data)


@require_GET
def task_upgrade_overview(request, machine_id, upgrade_id, task):
    machine = get_object_or_404(EventstatMachine, id=machine_id)
    upgrade = get_object_or_404(EventstatUpgrade, id=upgrade_id)
    tasks = EventstatDetail.objects.filter(
        publish=True,
        task=task,
        machine=machine,
        upgrade__date__lte=upgrade.date,
    ).values(
        'upgrade__date',
    ).annotate(
        average=models.Avg('total'),
        minimum=models.Min('total'),
        maximum=models.Max('total'),
        stddev=models.StdDev('total'),
    ).order_by(
        'upgrade__date',
    )
    last_release_avg = 0.0
    last_release_name = 'Unknown'
    bundle = {
        'last_release_avg': last_release_avg,
        'last_release_name': last_release_name,
    }
    task_data = []
    for t in tasks:
        task_data.append(
            dict(
                build_number=t['upgrade__date'].strftime("%Y%m%d"),
                bottom=t['minimum'],
                min=t['minimum'],
                max=t['maximum'],
                avg=t['average'],
                std=t['stddev'],
            ),
        )

    bundle['count'] = tasks.count()
    bundle['image_data'] = task_data
    data = json.dumps(bundle)

    return JSONResponse(data)
