# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import logging

from django.core.management.base import BaseCommand
from django.db import models

from eventstat.models import (
    EventstatResult,
    EventstatMetric,
)
from eventstat.management import get_task_types


class Command(BaseCommand):
    help = "Do eventstat detail aggregations."

    def _aggregate(self, task_type, image, upgrade, machine):
        """ Aggregate values by name. """

        for task_type in get_task_types():

            results = EventstatResult.objects.filter(
                publish=True,
                task_type=task_type,
                image=image,
                upgrade=upgrade,
                machine=machine,
            )

            if results.count() == 0:
                return

            ran_at = results[0].ran_at

            results = results.aggregate(
                average=models.Avg('value'),
                minimum=models.Min('value'),
                maximum=models.Max('value'),
                stddev=models.StdDev('value'),
            )

            metric, new_metric = EventstatMetric.objects.get_or_create(
                machine=machine,
                image=image,
                upgrade=upgrade,
                name=task_type,
                defaults=dict(
                    ran_at=ran_at,
                    average=results['average'],
                    minimum=results['minimum'],
                    maximum=results['maximum'],
                    stddev=results['stddev'],
                )
            )

            if not new_metric:
                metric.ran_at = ran_at
                metric.average = results['average']
                metric.minimum = results['minimum']
                metric.maximum = results['maximum']
                metric.stddev = results['stddev']
                metric.publish = True
                metric.save()

    def handle(self, *args, **options):

        verbosity = int(options.get('verbosity'))

        log_level = logging.WARN
        if verbosity == 2:
            log_level = logging.INFO
        if verbosity > 2:
            log_level = logging.DEBUG

        logging.basicConfig(level=log_level)

        results = EventstatResult.objects.filter(publish=True)
        results = results.distinct('task_type', 'image', 'upgrade', 'machine')

        for result in results.iterator():
            logging.info("task_type: {}".format(result.task_type))
            self._aggregate(
                result.task_type,
                result.image,
                result.upgrade,
                result.machine,
            )
