# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import logging

from django.core.management.base import BaseCommand
from django.db import models

from eventstat.models import (
    EventstatDetail,
    EventstatResult,
)
from eventstat.management import get_task_types


class Command(BaseCommand):
    help = "Do eventstat detail aggregations."

    def _aggregate(self, interval, image, upgrade, machine):
        """ Aggregate values by name. """

        for task_type in get_task_types():

            details = EventstatDetail.objects.filter(
                publish=True,
                interval=interval,
                image=image,
                upgrade=upgrade,
                machine=machine,
            )

            if details.count() == 0:
                return

            values = [
                'image',
                'upgrade',
                'machine',
                'interval',
            ]

            if task_type != "total":
                details = details.filter(task_type=task_type)
                values.append('task_type')

            if details.count() == 0:
                return

            ran_at = details[0].ran_at

            details = details.values(*values).annotate(
                models.Sum('total'),
            )

            for res in details:

                result, new_result = EventstatResult.objects.get_or_create(
                    machine=machine,
                    image=image,
                    upgrade=upgrade,
                    task_type=task_type,
                    interval=interval,
                    defaults=dict(
                        ran_at=ran_at,
                        value=res['total__sum'],
                    ),
                )

                if not new_result:
                    result.value = res['total__sum']
                    result.ran_at = ran_at
                    result.save()

    def handle(self, *args, **options):

        verbosity = int(options.get('verbosity'))

        log_level = logging.WARN
        if verbosity == 2:
            log_level = logging.INFO
        if verbosity > 2:
            log_level = logging.DEBUG

        logging.basicConfig(level=log_level)

        details = EventstatDetail.objects.filter(publish=True)
        details = details.distinct('interval', 'image', 'upgrade', 'machine')

        for detail in details.iterator():
            logging.info("interval: {}".format(detail.interval))
            self._aggregate(
                detail.interval,
                detail.image,
                detail.upgrade,
                detail.machine,
            )
