# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import csv
import logging
import os
import re

from django.core.management import call_command

from common.management import (
    get_kernel,
    jenkins_get,
    JenkinsBaseCommand,
)
from eventstat.models import (
    EventstatDetail,
    EventstatImage,
    EventstatLog,
    EventstatMachine,
    EventstatMetric,
    EventstatResult,
    EventstatUpgrade,
    TASK_TYPE_CHOICES,
)
from performance.management import (
    get_image_build_number,
    get_run_number,
)

from common.utils import split_build_number


def _extract_build_data(
    install_data,
    logs,
    unzip_path,
    online_path,
    log_name,
    build_date,
    kernel,
    image_build_number,
):

    log_path = "{}/{}".format(
        unzip_path,
        log_name,
    )
    log_path_url = "{}/{}".format(
        online_path,
        log_name,
    )
    logging.info("log_path_url: %s", log_path_url)

    install_type = install_data['method']
    dashboard_data = dict(
        release=install_data['release'],
        variant=install_data['variant'],
        arch=install_data['arch'],
        build_date=build_date,
        kernel=kernel,
    )
    if 'jenkins_build' in install_data:
        dashboard_data['jenkins_build'] = install_data['jenkins_build']

    if install_type == 'install':
        dashboard_data['build_number'] = image_build_number
    else:
        dashboard_data['date'] = build_date

        # XXX: if we start having builds that upgrade across
        # releases we will need to update this
        dashboard_data['from_release'] = install_data['release']

    # Assume we need to retrieve the logs if there's no 'unzip_path'
    if 'unzip_path' not in install_data:
        logging.warn("Getting logs from jenkins directly...")
        dashboard_data['output'] = jenkins_get(
            log_path_url,
            as_json=False,
        )
    else:
        dashboard_data['csv'] = log_path

    dashboard_data['log_path'] = log_path
    dashboard_data['log_path_url'] = log_path_url
    dashboard_data['log_name'] = log_name

    return dashboard_data


def _parse_data_row(
    data,
    mapping,
    row_key,
    row,
    interval,
    parse_value_func,
    task_type_func,
):
    field = mapping.get(row_key, row_key)

    # strip off row key
    row = row[1:]
    for i in range(len(row)):
        if len(data) <= i:
            data.insert(i, dict())

        value = row[i]

        # Add interval values to a sub-array.
        if interval is not None:
            if 'intervals' not in data[i]:
                data[i]['intervals'] = []

            data[i]['intervals'].append(
                dict(
                    interval=interval,
                    value=value,
                ),
            )
            continue

        if field == 'task':
            data[i]['command'] = value
            value = parse_value_func(value)
            logging.info("value: %s", value)
            data[i]['task_type'] = task_type_func(value)

        data[i][field] = value


class Command(JenkinsBaseCommand):

    job_regex = re.compile(
        ur"eventstat-([^-]+)-([^-]+)-([^-]+)-(install|upgrade)-([^-]+)-(.*)",
    )

    history_days = 30

    def _get_eventstat_logs(self, artifacts):
        """ Check for valid eventstat results.

        :returns: a list of eventstat logs.

        """

        logs = {}
        eventstat_log_regex = re.compile(
            ur'^(clientlogs\/|)(results\/|)(\d+\/|)eventstat.csv',
        )
        log_regex = re.compile(ur'^(clientlogs\/|)(results\/|)(\d+\/|)eventstat.(csv,log)')
        utah_log_regex = re.compile(ur'^(clientlogs\/|)(clientlogs\/|)utah.*.yaml')

        for artifact in artifacts:
            logging.info("artifact: %s", artifact)
            path = artifact['relativePath']
            num = get_run_number(path, prefix="results\/")

            if utah_log_regex.match(path):
                logging.debug("found %s", path)
                logs['utah'] = artifact

            if num is None:
                num = 1

            if num not in logs:
                logs[num] = {
                    'data_file': None,
                    'logs': [],
                }

            logging.debug("artifact: %s", path)
            if eventstat_log_regex.match(path):
                logging.debug("found %s", path)
                logs[num]['data_file'] = artifact
            if log_regex.match(path):
                logging.debug("found eventstat log %s", path)
                logs[num]['logs'].append(artifact)

        return logs

    def _process_logs(self, build, build_date, install_data, func):
        """ Process eventstat logs calling func() for each log. """

        if not hasattr(func, '__call__'):
            raise Exception("invalid callable: {}".format(func))

        unzip_path = self.get_artifacts(build)
        if unzip_path is None:
            return

        install_data['unzip_path'] = unzip_path

        logs = self._get_eventstat_logs(build['artifacts'])
        image_build_number = get_image_build_number(build)

        if image_build_number == "UNKNOWN":
            logging.warn("build_number: %s", image_build_number)
            return

        utah_log = logs.pop('utah', None)
        kernel = get_kernel(build['url'], utah_log)
        unzip_path = '{}artifact'.format(build['url'])
        online_path = unzip_path

        if 'unzip_path' in install_data:
            unzip_path = "{}/archive".format(install_data['unzip_path'])

        for num in logs:
            # skip incomplete logs
            if logs[num]['data_file'] is None:
                logging.info(
                    "missing data: %s",
                    logs[num]['data_file'],
                )
                continue

            log_name = logs[num]['data_file']['relativePath']

            dashboard_data = _extract_build_data(
                install_data,
                logs,
                unzip_path,
                online_path,
                log_name,
                build_date,
                kernel,
                image_build_number,
            )

            func(dashboard_data)

    def _parse_task(self, task):

        if task[0] == '[':
            return task.split()[0]

        return os.path.basename(task.split()[0])

    def _task_type(self, task):

        # task names enclosed in square brackets '[]' are
        # considered kernel tasks all others are userland.
        if task[0] == '[':
            return TASK_TYPE_CHOICES[0][1]
        else:
            return TASK_TYPE_CHOICES[1][1]

    def _parse_log(self, filename):
        """ Parse an eventstat.csv file. """

        mapping = {
            'Task': 'task',
            'Init Function': 'init_function',
            'Callback': 'callback',
            'Total': 'total',
            'Max': 'max',
            'Min': 'min',
            'Average': 'avg',
            'Std.Dev.': 'stddev',
        }

        data = []
        with open(filename, 'rb') as f:

            reader = csv.reader(f)

            for row in reader:
                row_key = row[0].rstrip(':')
                logging.info("row_key type: %s", type(row_key))

                try:
                    interval = int(float(row_key))
                except ValueError:
                    interval = None

                if row_key not in mapping and interval is None:
                    logging.warn("unknown row: %s", row_key)
                    continue

                _parse_data_row(
                    data,
                    mapping,
                    row_key,
                    row,
                    interval,
                    self._parse_task,
                    self._task_type,
                )

        return data

    def _valid_log_format(self, filename):
        """ Determine if an eventstat log is valid.
        Right now this means it has a 'Max:' row.

        """

        valid = False
        with open(filename, 'rb') as f:

            reader = csv.reader(f)

            for row in reader:
                if row[0].rstrip(':') == 'Max':
                    valid = True
                    break

        return valid

    def _get_image_or_upgrade(self, dashboard_data):
        """ Add appropriate image and upgrade. """

        if 'build_number' in dashboard_data:
            image, new_image = EventstatImage.objects.get_or_create(
                release=dashboard_data['release'],
                variant=dashboard_data['variant'],
                arch=dashboard_data['arch'],
                build_number=dashboard_data['build_number'],
                # fake an md5 since we don't have the data
                md5="{}{}{}{}".format(
                    self.arch,
                    self.release,
                    self.variant,
                    dashboard_data['build_number'],
                )
            )

            split_build_number(image)
            self.image = image
            self.upgrade = None
        else:  # upgrade
            upg, new_upgr = EventstatUpgrade.objects.get_or_create(
                release=dashboard_data['release'],
                variant=dashboard_data['variant'],
                arch=dashboard_data['arch'],
                date=dashboard_data['date'],
                from_release=dashboard_data['from_release'],
            )
            self.upgrade = upg
            self.image = None

    def _process_result_items(self, items, log, dashboard_data):
        """ Process result items adding results as needed. """
        for item in items:
            logging.info("item: %s", item)
            task_type = item['task_type']

            for interval in item.get('intervals', []):
                total = int(float(interval['value']))
                interval = int(float(interval['interval']))

                detail, new_detail = EventstatDetail.objects.get_or_create(
                    image=self.image,
                    upgrade=self.upgrade,
                    machine=self.machine,
                    ran_at=dashboard_data['build_date'],
                    interval=interval,
                    task=item['task'],
                    task_type=task_type,
                    init_function=item['init_function'],
                    callback=item['callback'],
                    total=total,
                    log=log,
                    jenkins_url=self.jenkins_url,
                )

    def add_result(self, dashboard_data):
        """ Add eventstat data to the database. """

        if not self._valid_log_format(dashboard_data['csv']):
            logging.info(
                "Invalid log format: %s",
                dashboard_data['log_path_url'],
            )
            return

        logging.info("Adding eventstat result.")

        items = self._parse_log(dashboard_data['csv'])

        self._get_image_or_upgrade(dashboard_data)

        log, new_log = EventstatLog.objects.get_or_create(
            path=dashboard_data['log_path_url'],
            name=dashboard_data['log_name'],
        )

        self.jenkins_url = ""
        if 'jenkins_build' in dashboard_data:
            self.jenkins_url = dashboard_data['jenkins_build'].url

        self._process_result_items(items, log, dashboard_data)

    def remove_result(self, dashboard_data):
        """ Remove eventstat data from the database. """

        logging.info("Removing eventstat result.")

        self._get_image_or_upgrade(dashboard_data)

        metrics = EventstatMetric.objects.filter(
            publish=True,
            machine=self.machine,
            image=self.image,
            upgrade=self.upgrade,
        )

        metrics.delete()

        results = EventstatResult.objects.filter(
            publish=True,
            machine=self.machine,
            image=self.image,
            upgrade=self.upgrade,
        )

        results.delete()

        details = EventstatDetail.objects.filter(
            publish=True,
            machine=self.machine,
            image=self.image,
            upgrade=self.upgrade,
        )

        details.delete()

        if self.image is not None:
            if self.image.metrics.count() == 0:
                self.image.delete()

        if self.upgrade is not None:
            if self.upgrade.metrics.count() == 0:
                self.upgrade.delete()

        logs = EventstatLog.objects.filter(
            path=dashboard_data['log_path_url'],
            name=dashboard_data['log_name'],
        )

        logs.delete()

    def extract_data(self, name):
        m = self.job_regex.match(name)

        if m:
            self.release = m.group(1)
            self.variant = m.group(2)
            self.arch = m.group(3)
            self.method = m.group(4)
            self.workload = m.group(5)
            self.machine_name = m.group(6)

            logging.info("name: %s", name)
            logging.info("method: %s", self.method)

            self.install_data = dict(
                release=self.release,
                variant=self.variant,
                arch=self.arch,
                method=self.method,
            )

    def process_job(self, job):
        logging.info("job.name: %s", job['name'])
        self.machine, new_machine = EventstatMachine.objects.get_or_create(
            name=self.machine_name,
        )

    def post_process_job(self, job):

        call_command('agg_results_eventstat')
        call_command('agg_metrics_eventstat')

        if self.machine.metrics.count() == 0:
            logging.info("removing machine %s", self.machine)
            self.machine.delete()
