# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.urlresolvers import reverse
from django.db import models
from django.shortcuts import (
    redirect,
    render_to_response,
    get_object_or_404,
)
from django.template import RequestContext
from django.views.decorators.http import require_GET


from django_tables2 import RequestConfig

from common.templatetags.dashboard_extras import percent

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)

from power.views import overview

from eventstat.models import (
    EventstatDetail,
    EventstatImage,
    EventstatMachine,
    EventstatMetric,
    EventstatUpgrade,
    TASK_TYPE_CHOICES,
)
from eventstat.tables import (
    TaskInstallTable,
    TaskUpgradeTable,
    MachineInstallDetailsTable,
    MachineUpgradeDetailsTable,
)
from eventstat.utils import (
    get_arches,
    get_machines,
    previous,
)

TASK_TYPES = ['all'] + [x[1] for x in TASK_TYPE_CHOICES]


def _generate_task_type_urls(url, kwargs, task_types=TASK_TYPES):

    retval = []

    for task_type in task_types:
        kwargs['task_type'] = task_type
        ret_url = reverse(
            url,
            kwargs=kwargs,
        )
        retval.append((task_type, ret_url))

    return retval


@BreadCrumb("Wakeups", parent=overview)
@require_GET
def arch_overview(request, arch=None):
    arches = get_arches()

    if arch is None:
        arch = arches[0] if len(arches) > 0 else 'amd64'
        return redirect(
            reverse(
                'eventstat_arch_overview',
                kwargs={
                    'arch': arch,
                }
            ),
        )

    machines = list(get_machines(arch))

    data = {
        'arches': arches,
        'machines': machines,
        'arch': arch,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(arch_overview),
    }

    return render_to_response(
        'eventstat/arch_overview.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Machine details",
    parent=arch_overview,
    needs=['arch', 'machine_id', 'method'],
)
@require_GET
def machine_details(request, method, arch, machine_id):
    machine = get_object_or_404(EventstatMachine, id=machine_id)

    metrics = EventstatMetric.objects.exclude(
        publish=False,
    ).filter(
        machine=machine_id,
        name='total',
    )

    if method == "install":
        metrics = metrics.filter(
            image__arch=arch,
        ).order_by(
            '-image__build_number',
        ).values(
            'image__build_number',
            'machine__id',
            'image__id',
            'average',
            'minimum',
            'maximum',
            'stddev',
        )
    else:
        metrics = metrics.filter(
            upgrade__arch=arch,
        ).order_by(
            '-upgrade__date',
        ).values(
            'upgrade__date',
            'machine__id',
            'upgrade__id',
            'average',
            'minimum',
            'maximum',
            'stddev',
        )

    table_data = []
    for metric in metrics:
        if method == "install":
            table_data.append(
                dict(
                    build_number=metric['image__build_number'],
                    image_id=metric['image__id'],
                    machine=metric['machine__id'],
                    average=metric['average'],
                    minimum=metric['minimum'],
                    maximum=metric['maximum'],
                    stddev=metric['stddev'],
                    pct_stddev=percent(metric['stddev'], metric['average']),
                ),
            )
            table = MachineInstallDetailsTable(table_data)
        else:
            table_data.append(
                dict(
                    upgrade_date=metric['upgrade__date'].strftime("%Y-%m-%d"),
                    upgrade_id=metric['upgrade__id'],
                    machine=metric['machine__id'],
                    average=metric['average'],
                    minimum=metric['minimum'],
                    maximum=metric['maximum'],
                    stddev=metric['stddev'],
                    pct_stddev=percent(metric['stddev'], metric['average']),
                ),
            )
            table = MachineUpgradeDetailsTable(table_data)

    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = {
        'arch': arch,
        'machine': machine,
        'table': table,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            machine_details,
            machine_id=machine_id,
            arch=arch,
            method=method,
        ),
    }
    if method == "install":
        data['image'] = True
    else:
        data['upgrade'] = True
    return render_to_response(
        'eventstat/machine_details.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "{build_number} details",
    parent=machine_details,
    needs=['image_id', 'machine_id', 'task_type'],
)
@require_GET
def image_details(request, image_id, task_type, machine_id):
    machine = get_object_or_404(EventstatMachine, id=machine_id)
    image = get_object_or_404(EventstatImage, id=image_id)

    results = EventstatDetail.objects.filter(
        publish=True,
        machine=machine,
        image=image,
    ).values(
        'task',
        'image_id',
        'image__arch',
        'machine_id',
    ).annotate(
        average=models.Avg('total'),
        minimum=models.Min('total'),
        maximum=models.Max('total'),
        stddev=models.StdDev('total'),
    )

    if task_type != 'all':
        results = results.filter(task_type=task_type)

    table_data = []
    for result in results:
        last = previous(
            machine=result['machine_id'],
            task=result['task'],
            image=image.build_number,
        )
        if last is None:
            last = 0
        result['change'] = result['average'] - last
        movement = "movement"
        if result['change'] > 0:
            movement = "up"
        elif result['change'] < 0:
            movement = "down"
        table_data.append(
            dict(
                image_id=result['image_id'],
                machine_id=result['machine_id'],
                task=result['task'],
                average=result['average'],
                minimum=result['minimum'],
                maximum=result['maximum'],
                stddev=result['stddev'],
                movement=movement,
                change=result['change'],
                pct_stddev=percent(result['stddev'], result['average']),
            ),
        )
    table = TaskInstallTable(table_data)

    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = {
        'table': table,
        'image': image,
        'task_type': task_type,
        'machine': machine,
        'method': "install",
        'task_type_urls': _generate_task_type_urls(
            'eventstat_image_details',
            kwargs=dict(
                machine_id=machine_id,
                image_id=image_id,
                task_type=task_type,
            ),
        ),
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_details,
            machine_id=machine_id,
            image_id=image_id,
            task_type=task_type,
            build_number=image.build_number,
            arch=image.arch,
            method='install',
        ),
    }

    return render_to_response(
        "eventstat/image_details.html",
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "{upgrade_date} upgrade details",
    parent=machine_details,
    needs=['upgrade_id', 'machine_id', 'task_type'],
)
@require_GET
def upgrade_details(request, upgrade_id, task_type, machine_id):
    machine = get_object_or_404(EventstatMachine, id=machine_id)
    upgrade = get_object_or_404(EventstatUpgrade, id=upgrade_id)

    results = EventstatDetail.objects.filter(
        publish=True,
        machine=machine,
        upgrade=upgrade,
    ).values(
        'task',
        'upgrade_id',
        'upgrade__arch',
        'machine_id',
    ).annotate(
        average=models.Avg('total'),
        minimum=models.Min('total'),
        maximum=models.Max('total'),
        stddev=models.StdDev('total'),
    )

    if task_type != 'all':
        results = results.filter(task_type=task_type)

    table_data = []
    for result in results:
        last = previous(
            machine=result['machine_id'],
            task=result['task'],
            upgrade=upgrade.date,
        )
        if last is None:
            last = 0
        result['change'] = result['average'] - last
        movement = "movement"
        if result['change'] > 0:
            movement = "up"
        elif result['change'] < 0:
            movement = "down"
        table_data.append(
            dict(
                upgrade_id=result['upgrade_id'],
                machine_id=result['machine_id'],
                task=result['task'],
                average=result['average'],
                minimum=result['minimum'],
                maximum=result['maximum'],
                stddev=result['stddev'],
                movement=movement,
                change=result['change'],
                pct_stddev=percent(result['stddev'], result['average']),
            ),
        )
    table = TaskUpgradeTable(table_data)

    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = {
        'table': table,
        'upgrade': upgrade,
        'task_type': task_type,
        'machine': machine,
        'method': "upgrade",
        'task_type_urls': _generate_task_type_urls(
            'eventstat_upgrade_details',
            kwargs=dict(
                machine_id=machine_id,
                upgrade_id=upgrade_id,
                task_type=task_type,
            ),
        ),
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            upgrade_details,
            machine_id=machine_id,
            upgrade_id=upgrade_id,
            task_type=task_type,
            upgrade_date=upgrade.date.strftime("%Y-%m-%d"),
            method='upgrade',
            arch=upgrade.arch,
        ),
    }

    return render_to_response(
        "eventstat/upgrade_details.html",
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "{task} overview",
    parent=upgrade_details,
    needs=['upgrade_id', 'machine_id', 'task'],
)
@require_GET
def upgrade_task_overview(request, upgrade_id, machine_id, task):
    machine = get_object_or_404(EventstatMachine, id=machine_id)
    upgrade = get_object_or_404(EventstatUpgrade, id=upgrade_id)

    task = EventstatDetail.objects.filter(
        publish=True,
        task=task,
        machine=machine,
        upgrade=upgrade,
    )[0]

    data = {
        'build_number': upgrade.date.strftime("%Y-%m-%d"),
        'upgrade': upgrade,
        'build': upgrade,
        'machine': machine,
        'task': task,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            upgrade_task_overview,
            machine_id=machine_id,
            upgrade_id=upgrade_id,
            upgrade_date=upgrade.date,
            arch=upgrade.arch,
            method='upgrade',
            task=task.task,
            task_type="all",
        ),
    }
    return render_to_response(
        "eventstat/task_overview.html",
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "{task} overview",
    parent=image_details,
    needs=['image_id', 'machine_id', 'task'],
)
@require_GET
def image_task_overview(request, image_id, machine_id, task):
    machine = get_object_or_404(EventstatMachine, id=machine_id)
    image = get_object_or_404(EventstatImage, id=image_id)

    task = EventstatDetail.objects.filter(
        publish=True,
        task=task,
        machine=machine,
        image=image,
    )[0]

    data = {
        'build_number': image.build_number,
        'image': image,
        'machine': machine,
        'build': image,
        'task': task,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_task_overview,
            machine_id=machine_id,
            image_id=image_id,
            build_number=image.build_number,
            task=task.task,
            task_type='all',
            arch=image.arch,
            method='install',
        ),
    }
    return render_to_response(
        "eventstat/task_overview.html",
        data,
        RequestContext(request),
    )
