# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version
# 3, as published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.


import logging
import re

from common.management import (
    JenkinsBaseCommand,
    get_kernel,
)

from common.utils import regexes

from performance.management import get_image_build_number

from common.utils import split_build_number

from idle_power.models import (
    IdlePowerImage,
    IdlePowerLog,
    IdlePowerMachine,
    IdlePowerBuild,
)

from idle_power.utah_utils import process_idle_power_log


def _get_power_logs(artifacts):
    """ Check for valid power results.

    :returns: a list of power logs.

    """
    logs = []
    utah_log_regex = re.compile(ur'^clientlogs\/utah.*.yaml')

    for artifact in artifacts:
        logging.debug(artifact['relativePath'])
        path = artifact['relativePath']
        logging.debug("found %s", path)
        if utah_log_regex.match(path):
            logs.append(artifact)

    return logs


def _process_utah_logs(
    logs,
    image,
    machine,
    jenkins_url=None,
    name=None,
):

    details = []

    for log in logs:
        logging.debug("log: %s", log)
        detail = process_idle_power_log(
            log,
            image,
            machine,
            jenkins_url=jenkins_url,
            name=name,
        )

        if detail:
            details.append(detail)

    return details


class Command(JenkinsBaseCommand):
    """ The command. """

    job_regex = regexes['idlepower']
    job_types = ['milestone', 'backfill']

    def extract_data(self, name):
        m = self.job_regex.match(name)

        if m:
            self.release = m.group(1)
            self.variant = m.group(2)
            self.arch = m.group(3)
            self.machine_name = m.group(4)

            self.install_data = dict(
                release=self.release,
                variant=self.variant,
                arch=self.arch,
                machine=self.machine_name,
            )

    def process_job(self, job):
        self.machine, new_machine = IdlePowerMachine.objects.get_or_create(
            name=self.machine_name,
        )

        if new_machine:
            logging.debug("Added new machine: %s", self.machine_name)

    def _process_logs(self, build, build_date, install_data, func):

        if not hasattr(func, '__call__'):
            raise Exception("invalid callable: %s", func)

        logs = _get_power_logs(build['artifacts'])
        self.image_build_number = get_image_build_number(build)
        self.build_url = build['url']
        for log in logs:
            kernel = get_kernel(build['url'], log)
            log_path = "{}artifact/{}".format(
                build['url'],
                log['relativePath'],
            )
            dashboard_data = dict(
                build_date=build_date,
                build_number=self.image_build_number,
                log=log,
                log_path=log_path,
                release=install_data['release'],
                variant=install_data['variant'],
                arch=install_data['arch'],
                kernel=kernel,
            )
            if 'jenkins_build' in install_data:
                dashboard_data['jenkins_build'] = install_data['jenkins_build']

            func(build, dashboard_data)

    def add_result(self, build, dashboard_data):
        build_logs = []
        for l in dashboard_data['log']:
            log, new_log = IdlePowerLog.objects.get_or_create(
                path=dashboard_data['log_path'],
                name=dashboard_data['log']['relativePath'],
            )

            if new_log:
                logging.info("Adding new log: %s", log)

            build_logs.append(log)

        self.image, new_image = IdlePowerImage.objects.get_or_create(
            release=dashboard_data['release'],
            variant=dashboard_data['variant'],
            arch=dashboard_data['arch'],
            build_number=dashboard_data['build_number'],
            # fake an md5 since we don't have the data
            md5="{}{}{}{}".format(
                self.arch,
                dashboard_data['release'],
                self.variant,
                dashboard_data['build_number'],
            )
        )
        split_build_number(self.image)

        if new_image:
            logging.debug("Added new image: %s", self.image)

        build, new_build = IdlePowerBuild.objects.get_or_create(
            image=self.image,
            machine=self.machine,
            build=dashboard_data['jenkins_build']
        )

        if len(build_logs) > 0:
            _process_utah_logs(
                build_logs,
                image=self.image,
                machine=self.machine,
                jenkins_url=self.build_url,
            )

    def remove_result(self, build, dashboard_data):
        """ Remove power data from the database """
        logging.info("Removing power result")

        try:
            image = IdlePowerImage.objects.get(
                arch=self.arch,
                release=self.release,
                variant=self.variant,
                build_number=self.image_build_number,
                md5="{}{}{}{}".format(
                    self.arch,
                    self.release,
                    self.variant,
                    self.image_build_number,
                )
            )

            image.delete()
        except IdlePowerImage.DoesNotExist:
            pass
