# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
UTAH result parsing code.
"""

import datetime
import logging
import time

from django.conf import settings

from common.utah_parser import UTAHParser, ParserError

from idle_power.models import (
    IdlePowerDetail,
)

FLAVOR = 'ubuntu'


def process_idle_power_log(
    logfile,
    image,
    machine,
    jenkins_url=None,
    name=None
):
    """
    Parse a utah client log for smoke test results.
    """
    details = []

    logfile_path = logfile
    if jenkins_url is not None:
        logfile_path = "{}/artifact/{}".format(
            jenkins_url,
            logfile,
        )

    if jenkins_url is None:
        jenkins_url = settings.JENKINS_URL

    parser = UTAHParser()

    try:
        data = parser.parse(logfile_path)
    except ParserError as e:
        if not e.zero_entry:
            logging.error("%s %s", logfile_path, e)
            return details

    if data is None:
        logging.warn("Unable to parse %s", logfile_path)
        return details

    # Use the name from the logs if there is one, if there isn't one
    # and one is passed in use that.
    if 'name' not in data:
        data['name'] = 'unnamed'

    if name is None or data['name'] != 'unnamed':
        name = data['name']
    data['machine'] = machine
    data['image'] = image
    data['jenkins_url'] = jenkins_url

    _add_details(details, data)


def _add_details(details, data):
    for command in data['commands']:
        if command['cmd_type'] != 'testcase_test':
            continue
        testcase = command['testcase']
        start_voltage = command['battery']['start']['voltage_now'] / 1000
        end_voltage = command['battery']['end']['voltage_now'] / 1000
        energy_start = command['battery']['start']['energy_now'] / 1000
        energy_end = command['battery']['end']['energy_now'] / 1000
        energy_used = energy_start - energy_end
        charge_start = command['battery']['start']['charge_now'] / 1000
        charge_end = command['battery']['end']['charge_now'] / 1000
        charge_used = charge_start - charge_end
        duration = command['time_delta']
        duration = time.strptime(duration.split('.')[0], '%H:%M:%S')
        duration = datetime.timedelta(
            hours=duration.tm_hour,
            minutes=duration.tm_min,
            seconds=duration.tm_sec).total_seconds()
        ran_at = command['start_time']
        voltage_est = (
            end_voltage + 0.5 * abs(start_voltage - end_voltage)
        ) / 1000
        calculated_energy = charge_used * voltage_est
        detail, new_detail = IdlePowerDetail.objects.get_or_create(
            image=data['image'],
            machine=data['machine'],
            jenkins_url=data['jenkins_url'],
            ran_at=ran_at,
            duration=duration,
            testcase=testcase,
            calculated_energy=calculated_energy,
            charge_start=charge_start,
            charge_used=charge_used,
            energy_start=energy_start,
            energy_used=energy_used,
        )
        details.append(detail)

    return details
