# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.views.decorators.http import require_GET
from django.template import RequestContext
from django.db.models import Avg, StdDev

from django.shortcuts import (
    render_to_response,
    get_object_or_404,
)

from django_tables2 import RequestConfig

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)

from power.views import overview

from idle_power.config import get_testcases

from idle_power.models import (
    IdlePowerMachine,
    IdlePowerDetail,
)

from idle_power.tables import DataTable, MachineTable


@BreadCrumb("Software measurement", parent=overview)
@require_GET
def arch_overview(request, arch=None):
    vals = IdlePowerDetail.objects.filter(
        publish=True,
        testcase__in=get_testcases(),
    ).values(
        'image__arch',
        'testcase',
        'machine__name',
        'machine__id',
    ).distinct()
    arches = [x['image__arch'] for x in vals]
    arches = sorted(arches)
    testcases = set(
        [x['testcase'] for x in vals if not arch or x['image__arch'] == arch]
    )

    chart_size = request.GET.get('size', '')

    if arch is None:
        if len(arches) > 0:
            arch = arches[0]
        else:
            arch = "UNKNOWN"

    result = vals.filter(image__arch=arch).distinct('machine__name')

    data = {
        'result': result,
        'arch': arch,
        'arches': set(arches),
        'active': arch,
        'chart_size': chart_size,
        'testcases': testcases,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(arch_overview),
    }

    return render_to_response(
        'idle_power/arch_overview.html',
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Machine details",
    parent=arch_overview,
    needs=['machine_id', 'arch'],
)
@require_GET
def machine_detail(request, machine_id, arch):
    machine = get_object_or_404(IdlePowerMachine, id=machine_id)
    qs = IdlePowerDetail.objects.filter(
        machine__id=machine_id
    ).values(
        'image__build_number',
        'image__arch',
        'testcase',
        'machine__id',
    ).annotate(
        energy_avg=Avg('energy_used'),
        energy_stddev=StdDev('energy_used'),
    ).order_by('-image__build_number')

    data = list(qs)
    for d in data:
        if d['energy_avg'] == 0:
            percent_stddev = 0
        else:
            percent_stddev = d['energy_stddev'] / d['energy_avg'] * 100

        d['energy_percent_stddev'] = percent_stddev

    table = MachineTable(data)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)
    data = {
        'name': machine.name,
        'arch': arch,
        'table': table,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            machine_detail,
            arch=arch,
            machine_id=machine_id,
        ),
    }

    return render_to_response(
        'idle_power/machine_table.html',
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Detailed view",
    parent=machine_detail,
    needs=['machine_id', 'arch', 'build_number'],
)
@require_GET
def machine_raw_data(request, machine_id, arch, build_number):
    machine = get_object_or_404(IdlePowerMachine, id=machine_id)

    qs = IdlePowerDetail.objects.filter(
        machine__id=machine_id,
        image__build_number=build_number,
        image__arch=arch,
    ).values(
        'image__build_number',
        'image__arch',
        'image__release',
        'image__variant',
        'ran_at',
        'testcase',
        'energy_start',
        'energy_used',
        'calculated_energy',
        'charge_start',
        'charge_used',
        'jenkins_url',
        'duration'
    )
    table = DataTable(qs)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)
    details = qs[0]
    data = {
        'name': machine.name,
        'id': machine.id,
        'arch': details['image__arch'],
        'table': table,
        'build_number': build_number,
        'release': details['image__release'],
        'ran_at': details['ran_at'],
        'variant': details['image__variant'],
        'jenkins_url': details['jenkins_url'],
        'duration': details['duration'],
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            machine_raw_data,
            arch=arch,
            machine_id=machine_id,
            build_number=build_number,
        ),
    }
    return render_to_response(
        "idle_power/machine_raw_data.html",
        data,
        RequestContext(request)
    )
