# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.


import json
import logging
import os
import re

from django.db import models

from performance.management import (
    get_image_build_number,
    get_run_number,
)

from common.management import (
    get_kernel,
    jenkins_get,
    JenkinsBaseCommand,
)

from memevent.models import (
    MemoryEventDetail,
    MemoryEventImage,
    MemoryEventMachine,
)

from common.utils import split_build_number


class Command(JenkinsBaseCommand):

    job_regex = re.compile(
        ur"memevent-([^-]+)-([^-]+)-([^-]+)-([^-]+)-(.*)",
    )

    def _get_memevent_logs(self, artifacts):
        """ Check for valid memevent results.

        :returns: a list of memevent logs.

        """

        logs = {}
        smem_log_regex = re.compile(
            ur'^(clientlogs\/|results\/|artifacts\/|)(\d+\/|)memory[-_]usage.json'
        )
        log_regex = re.compile(
            ur'^(clientlogs\/|results\/|artifacts\/|)(\d+\/|)memory[-_]usage.json'
        )
        utah_log_regex = re.compile(ur'^(clientlogs\/|)utah.*.yaml')

        for artifact in artifacts:
            path = artifact['relativePath']
            num = get_run_number(path, prefix="results\/")
            logging.debug("num: %s", num)

            if utah_log_regex.match(path):
                logging.debug("found %s", path)
                logs['utah'] = artifact

            if num is None:
                num = 1

            if num not in logs:
                logs[num] = {
                    'data_file': None,
                    'logs': [],
                }

            logging.debug("artifact: %s", path)
            if smem_log_regex.match(path):
                logging.debug("found %s", path)
                logs[num]['data_file'] = artifact
            if log_regex.match(path):
                logging.debug("found memevent log %s", path)
                logs[num]['logs'].append(artifact)

        return logs

    def verify(self, reading):
        retval = True

        keys = ['event', 'data']

        for key in keys:
            if key not in reading:
                retval = False
                break

        return retval

    def parse(self, data):

        key = 'readings'

        if key not in data:
            return None

        results = dict()

        event_index = 0
        for reading in data[key]:
            if self.verify(reading):
                event = reading['event']
                event_data = reading['data']
                results.setdefault(event, {})['event_index'] = event_index

                for pid, process in event_data.items():
                    results[event].setdefault('processes', []).append(process)

            event_index += 1

        return results

    def _parse_log(self, dashboard_data):
        """ Parse an smem.log file. """

        logging.debug("dashboard_data: %s", dashboard_data)

        # TODO: parse the josn here
        lines = dashboard_data['output'].split('\n')

        items = []

        count = 0
        for line in lines:
            count += 1

            # skip leading header line.
            if count == 1:
                continue

            if line == '':
                continue

            logging.debug("line: %s", line)
            data = line.split('\t')
            logging.debug("data: %s", data)
            try:
                items.append(
                    dict(
                        name=data[0],
                        command=data[1],
                        uss=int(data[2]),
                        pss=int(data[3]),
                        rss=int(data[4]),
                        vss=int(data[5]),
                    ),
                )
            except ValueError as e:
                logging.error("%s, log: %s", e, dashboard_data['log_path_url'])
                continue

        return items

    def add_result(self, dashboard_data):
        """ Add memevent data to the database. """

        logging.info("Adding memevent result.")

        data = self.parse(dashboard_data['data'])

        if 'build_number' in dashboard_data:
            image, new_image = MemoryEventImage.objects.get_or_create(
                release=dashboard_data['release'],
                variant=dashboard_data['variant'],
                arch=dashboard_data['arch'],
                build_number=dashboard_data['build_number'],
                # fake an md5 since we don't have the data
                md5="{}{}{}{}".format(
                    self.arch,
                    self.release,
                    self.variant,
                    dashboard_data['build_number'],
                )
            )
            split_build_number(image)
            self.image = image

            for event, items in data.items():
                event_index = items['event_index']
                for item in items['processes']:
                    logging.debug("item: %s", item)

                    md, new_md = MemoryEventDetail.objects.get_or_create(
                        image=self.image,
                        machine=self.machine,
                        user=item['user'],
                        command=item['command'],
                        uss=item['uss'],
                        pss=item['pss'],
                        rss=item['rss'],
                        swap=item['swap'],
                        pid=item['pid'],
                        ran_at=dashboard_data['build_date'],
                        event=event,
                        run=dashboard_data['run_number'],
                        jenkins_build=dashboard_data.get(
                            'jenkins_build',
                            None,
                        ),
                        defaults=dict(
                            event_index=event_index,
                        ),
                    )
                    if not new_md:
                        md.event_index = event_index
                        md.save()

            # only show the latest run for an image.
            details = MemoryEventDetail.objects.filter(
                image=self.image,
                machine=self.machine,
            ).aggregate(
                max_ran_at=models.Max('ran_at'),
            )

            max_ran_at = details['max_ran_at']

            details = MemoryEventDetail.objects.filter(
                image=self.image,
                machine=self.machine,
                ran_at=max_ran_at,
            ).aggregate(
                max_run=models.Max('run'),
            )

            max_run = details['max_run']

            if max_run is not None and max_ran_at is not None:
                MemoryEventDetail.objects.filter(
                    image=self.image,
                    machine=self.machine,
                ).exclude(
                    run=max_run,
                    ran_at=max_ran_at,
                ).update(publish=False)

            MemoryEventDetail.objects.filter(
                image=self.image,
                machine=self.machine,
                run=max_run,
                ran_at=max_ran_at,
            ).update(publish=True)

    def remove_result(self, dashboard_data):
        """ Remove memevent data from the database. """

        logging.info("Removing memevent result.")

        data = self.parse(dashboard_data['data'])

        image = None
        images = MemoryEventImage.objects.filter(
            release=dashboard_data['release'],
            variant=dashboard_data['variant'],
            arch=dashboard_data['arch'],
            build_number=dashboard_data['build_number'],
        )

        if images.count() > 0:
            image = images[0]

        for event, items in data.items():
            for item in items['processes']:
                logging.debug("item: %s", item)
                logging.debug("item['uss']: %s", item['uss'])

                details = MemoryEventDetail.objects.filter(
                    image=image,
                    machine=self.machine,
                    user=item['user'],
                    command=item['command'],
                    uss=item['uss'],
                    pss=item['pss'],
                    rss=item['rss'],
                    swap=item['swap'],
                    pid=item['pid'],
                    ran_at=dashboard_data['build_date'],
                    event=event,
                    run=dashboard_data['run_number'],
                )

                details.delete()

                if (
                    image is not None and
                    image.id is not None and
                    image.memoryeventdetail_set.count() == 0
                ):
                    image.delete()

    def _process_logs(self, build, build_date, install_data, func):
        """ Process memevent logs calling func() for each log. """

        if not hasattr(func, '__call__'):
            raise Exception("invalid callable: {}".format(func))

        unzip_path = self.get_artifacts(build)

        if unzip_path is None:
            return

        install_data['unzip_path'] = unzip_path

        logs = self._get_memevent_logs(build['artifacts'])
        logging.debug("logs: %s", logs)

        image_build_number = get_image_build_number(build)

        if image_build_number == "UNKNOWN":
            return

        utah_log = logs.pop('utah', None)

        kernel = get_kernel(build['url'], utah_log)

        unzip_path = '{}artifact'.format(build['url'])
        online_path = unzip_path

        if 'unzip_path' in install_data:
            unzip_path = "{}/archive".format(install_data['unzip_path'])

        for num in logs:
            # skip incomplete logs
            if logs[num]['data_file'] is None:
                logging.warn(
                    "missing data: %s",
                    build['url']
                )
                continue
            log_path = "{}/{}".format(
                unzip_path,
                logs[num]['data_file']['relativePath'],
            )
            log_path_url = "{}/{}".format(
                online_path,
                logs[num]['data_file']['relativePath'],
            )
            log_name = logs[num]['data_file']['relativePath']

            dashboard_data = dict(
                release=install_data['release'],
                variant=install_data['variant'],
                arch=install_data['arch'],
                build_date=build_date,
                kernel=kernel,
                run_number=num,
            )
            if 'jenkins_build' in install_data:
                dashboard_data['jenkins_build'] = install_data['jenkins_build']

            dashboard_data['build_number'] = image_build_number

            # Assume we need to retrieve the logs if there's no 'unzip_path'
            if 'unzip_path' not in install_data:
                logging.warn("Getting logs from jenkins directly...")
                dashboard_data['data'] = jenkins_get(
                    log_path_url,
                )
            else:
                if not os.path.exists(log_path):
                    logging.info("missing log file: %s", log_path)
                    continue

                output = self.get_local_file(log_path)

                if output == "":
                    logging.warn("Blank log file: %s", log_path_url)
                    continue

                dashboard_data['data'] = json.loads(
                    output,

                )

            dashboard_data['log_path'] = log_path
            dashboard_data['log_path_url'] = log_path_url
            dashboard_data['log_name'] = log_name

            func(dashboard_data)

    def extract_data(self, name):
        m = self.job_regex.match(name)

        if m:
            self.release = m.group(1)
            self.variant = m.group(2)
            self.arch = m.group(3)
            self.workload = m.group(4)
            self.machine_name = m.group(5)

            logging.debug("name: %s", name)

            self.install_data = dict(
                release=self.release,
                variant=self.variant,
                arch=self.arch,
            )

    def process_job(self, job):
        self.machine, new_machine = MemoryEventMachine.objects.get_or_create(
            name=self.machine_name,
        )
