# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.db import models
from django.shortcuts import (
    get_object_or_404,
    render_to_response,
)
from django.template import RequestContext
from django.views.decorators.http import require_GET

from django_tables2 import RequestConfig

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)
from common.views import index

from memevent.models import (
    MemoryEventDetail,
    MemoryEventImage,
    MemoryEventMachine,
    MemoryEventThreshold,
)
from memevent.tables import (
    DetailTable,
    ImageOverviewTable,
    ArchOverviewTable,
    EventDetailTable,
)


def _get_arches():
    arches = [x['image__arch'] for x in MemoryEventDetail.objects.filter(
        publish=True,
    ).values('image__arch').distinct()]

    return sorted(arches)


def _get_details(arch):
    details = list(
        MemoryEventDetail.objects.filter(
            publish=True,
            image__arch=arch,
        ).select_related('image', 'machine', 'jenkins_build')
        .values(
            'image__build_number',
            'image__id',
            'event',
            'run',
            'pss',
            'image__release',
            'image__variant',
            'image__arch',
            'machine__name',
            'machine__id',
            'jenkins_build__failed',
        )
    )

    _details = {}
    for detail in details:
        build_number = detail['image__build_number']
        image_id = detail['image__id']
        event = detail['event']
        run = detail['run']
        pss = detail['pss']
        arch = detail['image__arch']
        release = detail['image__release']
        variant = detail['image__variant']
        machine = detail['machine__name']
        status = "Failed"
        failed = detail['jenkins_build__failed']
        if not failed:
            status = 'Succeeded'

        # ensure there are defaults
        image = _details.setdefault(
            machine,
            {},
        ).setdefault(
            variant,
            {},
        ).setdefault(
            build_number,
            {},
        )

        image['failed'] = failed
        image['status'] = status
        image['release'] = release

        image.setdefault(
            'events', {},
        ).setdefault(event, {}).setdefault('runs', {})

        image['id'] = image_id
        image['arch'] = arch
        image['events'][event]['runs'].setdefault(run, 0)
        image['events'][event]['runs'][run] += pss
        image['events'][event]['machine_id'] = detail['machine__id']

    return _details


@BreadCrumb("Architecture Overview", parent=index)
@require_GET
def arch_overview(request, arch=None):

    arches = _get_arches()

    if arch is None:
        if len(arches) > 0:
            arch = arches[0]
        else:
            arch = "UNKNOWN"

    _details = _get_details(arch)

    thresholds = MemoryEventThreshold.objects.filter(
        publish=True,
    ).values()

    threshold_map = {}
    for threshold in thresholds:
        threshold_map.setdefault(
            threshold['machine__name'],
            {},
        ).setdefault(
            threshold['image__id'],
            [],
        ).append(threshold['threshold'])

    # calculate averages
    tables = {}
    for machine, m_data in _details.items():
        results = []
        for variant, v_data in m_data.items():
            for build_number, bn_data in v_data.items():
                events = ""
                for event, ev_data in bn_data['events'].items():
                    ev_data['average'] = 0
                    values = ev_data['runs'].values()

                    if len(values) > 0:
                        ev_data['average'] = float(sum(values)) / len(values)

                    events += "  {} ({:.2f}M)".format(
                        event,
                        ev_data['average'] / 1024,
                    )
                failed = bn_data['failed']
                status = bn_data['status']
                image_id = bn_data['id']

                thresholds = []

                if (
                    machine in threshold_map and
                    image_id in threshold_map[machine]
                ):
                    thresholds = threshold_map[machine][image_id]

                over_threshold = False

                if len(thresholds) > 0:
                    average = ev_data['average']
                    over_threshold = thresholds[0].threshold < average
                    threshold = ev_data['average'] = thresholds[0]['threshold']

                if not over_threshold:
                    threshold = ""

                results.append(
                    dict(
                        id=bn_data['id'],
                        build_number=build_number,
                        events=events,
                        release=bn_data['release'],
                        variant=variant,
                        machine=machine,
                        status=status,
                        failed=failed,
                        threshold=threshold,
                    ),
                )

        table = ArchOverviewTable(results, prefix='{}'.format(machine))
        RequestConfig(request, paginate={"per_page": 7}).configure(table)
        tables[machine] = table

    data = dict(
        tables=tables,
        arch=arch,
        arches=arches,
        bread_crumb_trail=BreadCrumbTrail.leading_to(arch_overview),
    )

    return render_to_response(
        'memevent/arch_overview.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Image Overview",
    parent=arch_overview,
    needs=['image_id', 'machine'],
)
@require_GET
def image_overview(request, image_id, machine):

    image = get_object_or_404(MemoryEventImage, id=image_id)
    machine = get_object_or_404(MemoryEventMachine, name=machine)

    details = MemoryEventImage.objects.filter(
        publish=True,
        id=image_id,
        memoryeventdetail__machine=machine,
    ).values(
        'id',
        'arch',
        'release',
        'variant',
        'build_number',
        'memoryeventdetail__event',
        'memoryeventdetail__run',
        'memoryeventdetail__machine__name',
    ).annotate(
        pss_sum=models.Sum('memoryeventdetail__pss'),
    )

    _details = {}
    for detail in details:
        build_number = detail['build_number']
        image_id = detail['id']
        event = detail['memoryeventdetail__event']
        run = detail['memoryeventdetail__run']
        pss_sum = detail['pss_sum']
        release = detail['release']
        variant = detail['variant']
        machine = detail['memoryeventdetail__machine__name']

        # ensure there are defaults
        image_data = _details.setdefault(build_number, {})
        image_data.setdefault(
            'events', {},
        ).setdefault(event, {}).setdefault('runs', {})

        image_data['id'] = image_id
        image_data['events'][event]['runs'][run] = pss_sum

    results = []
    averages = {}
    # calculate averages
    for build_number, bn_data in _details.items():
        events = ""
        for event, ev_data in bn_data['events'].items():
            ev_data['average'] = 0
            values = ev_data['runs'].values()

            if len(values) > 0:
                ev_data['average'] = float(sum(values)) / len(values)

            events += "  {} ({:.2f}M)".format(event, ev_data['average'] / 1024)
            averages.setdefault(build_number, []).append(
                dict(
                    event=event,
                    average=ev_data['average'],
                    id=bn_data['id'],
                    machine=machine,
                )
            )

        results.append(
            dict(
                id=bn_data['id'],
                build_number=build_number,
                events=events,
                release=release,
                variant=variant,
                machine=machine,
            ),
        )

    table = ImageOverviewTable(averages[image.build_number])
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = dict(
        table=table,
        image=image,
        machine=machine,
        bread_crumb_trail=BreadCrumbTrail.leading_to(
            image_overview,
            image_id=image_id,
            machine=machine,
        ),
    )

    return render_to_response(
        'memevent/image_overview.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Image Details",
    parent=image_overview,
    needs=['image_id', 'machine'],
)
@require_GET
def image_detail(request, image_id, machine):

    image = get_object_or_404(MemoryEventImage, id=image_id)
    machine = get_object_or_404(MemoryEventMachine, name=machine)

    details = MemoryEventDetail.objects.filter(
        publish=True,
        image=image,
        machine=machine,
    )

    table = DetailTable(details)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = dict(
        table=table,
        image=image,
        machine=machine,
        bread_crumb_trail=BreadCrumbTrail.leading_to(
            image_detail,
            image_id=image_id,
            machine=machine,
        ),
    )

    return render_to_response(
        'memevent/image_detail.html',
        data,
        RequestContext(request),
    )


@BreadCrumb(
    "Event Details",
    parent=image_detail,
    needs=['image_id', 'machine', 'event'],
)
@require_GET
def event_detail(request, image_id, machine, event):

    image = get_object_or_404(MemoryEventImage, id=image_id)
    machine = get_object_or_404(MemoryEventMachine, name=machine)

    details = list(
        MemoryEventDetail.objects.filter(
            publish=True,
            image=image,
            machine=machine,
            event=event,
        ).select_related('jenkins_build').order_by('run').values(
            'run',
            'command',
            'pss',
            'pss_threshold',
            'uss',
            'rss',
            'swap',
            'user',
            'jenkins_build__job__url',
            'jenkins_build__build_number',
        )
    )

    for detail in details:
        pss = detail['pss']
        pss_threshold = detail['pss_threshold']
        detail['over_pss_threshold'] = ""

        if pss > pss_threshold:
            detail['over_pss_threshold'] = pss - pss_threshold

    table = EventDetailTable(details)
    RequestConfig(request, paginate={"per_page": 100}).configure(table)

    data = dict(
        table=table,
        image=image,
        machine=machine,
        event=event,
        bread_crumb_trail=BreadCrumbTrail.leading_to(
            event_detail,
            image_id=image_id,
            machine=machine,
            event=event,
        ),
    )

    return render_to_response(
        'memevent/event_detail.html',
        data,
        RequestContext(request),
    )
