# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.test.client import Client

from power.models import (
    PowerImage,
    PowerLog,
    PowerMachine,
    PowerMetric,
)


class TestPower(TestCase):
    """ Tests for power models. """

    def setUp(self):
        """ Setup commonly used objects. """

        self.ran_at = datetime.datetime.now()
        self.log_count = 10

        self.image = PowerImage.objects.create(
            release='raring',
            variant='desktop',
            arch='amd64',
            md5='fakemd5',
            build_number='20130101',
        )

        self.assert_(self.image)

        self.machine = PowerMachine.objects.create(
            name='Test Machine',
            mac_address='00:00:00:00:01',
        )

        self.assert_(self.machine)

        self.metric = PowerMetric.objects.create(
            image=self.image,
            machine=self.machine,
            name="test_metric",
            ran_at=self.ran_at,
            minimum=0.1,
            maximum=0.2,
            average=0.15,
            stddev=0.02,
        )

        self.assert_(self.metric)

        self.logs = []
        for i in range(self.log_count):
            log = PowerLog.objects.create(
                path="http://jenkins.qa.ubuntu.com/job/log/{}".format(i),
                name="{}".format(i),
            )

            self.assert_(log)
            self.logs.append(log)
            self.metric.logs.add(log)

    def test_metric(self):
        """ Test PowerMetric creation. """

        self.assertEqual(self.metric.image, self.image)
        self.assertEqual(self.metric.machine, self.machine)
        self.assertEqual(self.metric.ran_at, self.ran_at)

    def test_machine_metric_count(self):
        """ Test machine returns the proper metric count. """

        self.assertEqual(self.machine.metric_count, 1)

    def test_image_metric_count(self):
        """ Test image returns the proper metric count. """

        self.assertEqual(self.image.metric_count, 1)

    def test_machine_metrics(self):
        """ Test machine returns proper list of metrics. """

        self.assertEqual(self.machine.metrics[0], self.metric)

    def test_image_metrics(self):
        """ Test image returns proper list of metrics. """

        self.assertEqual(self.image.metrics[0], self.metric)

    def test_logs(self):
        """ Test logs. """

        self.assertEqual(len(self.logs), self.log_count)

    def test_metric_log_count(self):
        """ Test that metric returns correct log count. """

        self.assertEqual(self.metric.log_count, self.log_count)


class TestPowerUrls(TestCase):
    """ Test the power app urls """

    c = Client()

    def setUp(self):
        """ Setup commonly used objects. """

        self.ran_at = datetime.datetime.now()
        self.image = PowerImage.objects.create(
            release='raring',
            variant='desktop',
            arch='amd64',
            md5='fakemd5',
            build_number='20130101',
        )

        self.assert_(self.image)

        self.machine = PowerMachine.objects.create(
            name='Test Machine',
            mac_address='00:00:00:00:01',
        )

        self.assert_(self.machine)

        self.metric = PowerMetric.objects.create(
            image=self.image,
            machine=self.machine,
            name="test_metric",
            ran_at=self.ran_at,
            minimum=0.1,
            maximum=0.2,
            average=0.15,
            stddev=0.02,
        )

        self.assert_(self.metric)

    def test_power_overview_url(self):
        overview = self.c.get(reverse('power_overview', ), )
        self.assertEqual(overview.status_code, 302)
        args = [self.image.arch, ]
        response = reverse('power_arch_overview', args=args)
        self.assertRedirects(overview, response)
        response = self.c.get(response)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'power/arch_overview.html')

    def test_power_arch_overiew_url(self):
        args = [self.image.arch, ]
        response = self.c.get(reverse('power_arch_overview', args=args, ), )
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'power/arch_overview.html')

    def test_power_machine_detail_url(self):
        args = [self.machine.id, self.image.arch]
        response = self.c.get(
            reverse(
                'power_machine_detail',
                args=args,
            ),
        )
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'power/machine_detail.html')

    def test_power_api_images_url(self):
        response = self.c.get(reverse('power_api_images'))
        self.assertEquals(response.status_code, 200)

    def test_power_api_machines_url(self):
        response = self.c.get(reverse('power_api_machines'))
        self.assertEquals(response.status_code, 200)

    def test_power_api_images_by_arch_url(self):
        args = [self.image.arch, ]
        response = self.c.get(reverse('power_api_images_by_arch', args=args), )
        self.assertEqual(response.status_code, 200)

    def test_power_api_image_detail_url(self):
        args = [self.image.id, ]
        response = self.c.get(reverse('power_api_image_detail', args=args), )
        self.assertEqual(response.status_code, 200)

    def test_power_api_image_machine_metrics_url(self):
        args = [self.image.id, self.machine.id]
        response = self.c.get(
            reverse(
                'power_api_image_machine_metrics',
                args=args,
            ),
        )
        self.assertEqual(response.status_code, 200)

    def test_power_api_machine_metric_results(self):
        args = [self.machine.id, self.image.arch, self.metric.name]
        response = self.c.get(
            reverse(
                'power_api_machine_metric_results',
                args=args,
            ),
        )
        self.assertEqual(response.status_code, 200)
