# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include, patterns, url
from django.core.management import call_command
from django.core.urlresolvers import reverse_lazy
from django.db import models
from django.template.loader import render_to_string

from common.plugin_helper import (
    Extension,
    KPI,
)

from smokeng.models import (
    SmokeResult,
    SmokeResultBug,
)
from smokeng.tables import SmokeKPITable

KPI_RELEASES = ['precise', 'trusty']
KPI_VARIANT_EXCLUDES = ['touch_sf4p', 'touch_custom_demo']


class SmokengPassRateKPI(KPI):
    """
    The original KPI entry that shows more data for each release.
    This class is used as the base class for the versions of the KPI
    that followed.

    """

    @classmethod
    def name(self):
        return "Image Testing Pass Rates"

    def importance(self):
        return self.IMPORTANCE_HIGH

    def _get_result_bug_count(self, build_number, variant=None):
        qs = SmokeResultBug.objects.filter(
            publish=True,
            result__image__rootfs_id=build_number,
            result__image__variant=variant,
        ).values(
            'result__image__release',
            'bug__bug_no',
        ).annotate(
            models.Count('bug__bug_no'),
        ).values(
            'bug__bug_no',
            'result__image__release',
            'bug__bug_no__count',
        )

        result = dict()
        for bug in qs:
            result.setdefault(
                bug['result__image__release'],
                0,
            )
            result[bug['result__image__release']] += 1

        return result

    def _get_result_data(self):
        qs = SmokeResult.objects.filter(
            publish=True,
            image__release__in=KPI_RELEASES,
            status__in=(SmokeResult.COMPLETE, SmokeResult.SYNCING),
        ).exclude(
            image__build_number__contains='?',
        ).exclude(
            image__variant__in=KPI_VARIANT_EXCLUDES,
        ).values(
            'image__arch',
            'image__variant',
            'image__release',
            'image__build_number',
            'image__rootfs_id',
        ).annotate(
            avg_pass_rate=models.Avg('pass_rate'),
        ).order_by('-image__rootfs_id')

        return qs

    def _get_entry_tree(self):
        entries = self._get_result_data()

        entry_tree = {}
        for entry in entries:
            build = entry_tree.setdefault(
                entry['image__release'],
                {},
            ).setdefault(
                entry['image__variant'],
                {},
            ).setdefault(
                entry['image__arch'],
                {},
            ).setdefault(
                entry['image__rootfs_id'],
                {},
            )

            build.setdefault('pass_rates', []).append(entry['avg_pass_rate'])
            build.setdefault('build_number', entry['image__build_number'])

        return entry_tree

    def _calculate_pass_rates(self):

        entry_tree = self._get_entry_tree()
        TREND_ENTRIES = 7

        results = {}
        for release, variants in entry_tree.items():
            for variant, arches in variants.items():
                for arch_name, arch in arches.items():
                    res_arch = results.setdefault(
                        release,
                        {},
                    ).setdefault(
                        variant,
                        {},
                    ).setdefault(
                        arch_name,
                        {},
                    )
                    keys = arch.keys()

                    item_count = min(len(keys), TREND_ENTRIES)
                    for build in sorted(keys, reverse=True)[:item_count]:
                        res_arch.setdefault(
                            'latest_build_number',
                            build
                        )
                        arch_build = arch[build]
                        arch_build['pass_rate'] = 0
                        if len(arch_build['pass_rates']) > 0:
                            pr_sum = sum(arch_build['pass_rates'])
                            pr_len = len(arch_build['pass_rates'])
                            arch_build['pass_rate'] = pr_sum / pr_len

                        res_arch.setdefault(
                            'pass_rate',
                            arch_build['pass_rate'],
                        )
                        res_arch.setdefault(
                            'full_build_number',
                            arch_build['build_number'],
                        )

                        if build > res_arch['latest_build_number']:
                            res_arch['latest_build_number'] = build
                            res_arch['pass_rate'] = arch_build['pass_rate']

                        res_arch.setdefault('data', []).append(
                            dict(
                                release=release,
                                arch=arch_name,
                                build_number=build,
                                pass_rate=arch_build['pass_rate'],
                            )
                        )

        return results

    def render(self):

        results = self._calculate_pass_rates()

        tables = []
        for release in sorted(results.keys(), reverse=True):
            arches = results[release]
            entries = []

            for arch in sorted(arches.keys()):
                data = arches[arch]
                entries.append(
                    dict(
                        release=release,
                        arch=arch,
                        build_number=data['latest_build_number'],
                        pass_rate=data['pass_rate'] * 100,
                    )
                )

            tables.append(
                dict(
                    release=release,
                    table=SmokeKPITable(
                        entries,
                        sortable=False,
                        prefix=release,
                    ),
                )
            )

        retval = render_to_string(
            'smokeng/kpi_pass_rate.html',
            dict(
                tables=tables,
            ),
        )

        return retval


class SmokengVariantPassRateKPI(SmokengPassRateKPI):
    """
    A simple KPI entry with an average pass-rate for each variant in each
    release.

    This KPI determines the latest build number for each variant and shows
    an average of those results for each release.

    To identify missing images a list of architectures used to calculate
    the pass-rate for each release are listed.

    A count of bugs filed by the QA Team is also included.

    """

    def _get_latest_build_number(self, results, release, variant):

        build_numbers = []

        if release in results and variant in results[release]:
            for arch, a_data in results[release][variant].items():
                build_numbers.append(a_data['latest_build_number'])

        if len(build_numbers) == 0:
            return ""

        # TODO: fix so it's sorted by float(build_number)
        return sorted(build_numbers, reverse=True)[0]

    def render(self):

        results = self._calculate_pass_rates()

        entries = []
        for release in sorted(results.keys(), reverse=True):
            variants = results[release]
            variant_entries = []
            for variant in sorted(variants.keys(), reverse=True):
                found_arches = []
                arches = results[release][variant]
                expected_build = self._get_latest_build_number(
                    results,
                    release,
                    variant,
                )

                bug_counts = self._get_result_bug_count(
                    expected_build,
                    variant,
                )

                arch_passrates = []
                has_expected_build = False
                for arch in sorted(arches.keys()):
                    data = arches[arch]
                    full_build_number = data['full_build_number']
                    build_no = data['latest_build_number']
                    if build_no == expected_build:
                        arch_passrates.append(data['pass_rate'])
                        found_arches.append(arch)
                        has_expected_build = True

                pass_rate = 0.0

                if len(arch_passrates) > 0:
                    pass_rate = sum(arch_passrates) / len(arch_passrates)

                bug_count = bug_counts.get(release, 0)

                if has_expected_build:
                    build_no = expected_build
                else:
                    build_no = ''

                variant_kpi = dict(
                    variant=variant,
                    results=dict(
                        build_no=build_no,
                        full_build_number=full_build_number,
                        pass_rate=pass_rate,
                        found_arches=found_arches,
                        bug_count=bug_count,
                    ),
                )
                variant_entries.append(variant_kpi)
            release_kpi = dict(
                release=release,
                kpi=variant_entries,
            )
            entries.append(release_kpi)

        retval = render_to_string(
            'smokeng/kpi_variant_pass_rate.html',
            dict(
                entries=entries,
                help_info="""The average pass_rate for each variant is
                calculated by accounting for each architecture (i.e. "amd64",
                "i386", "armhf") in the variant.  These averages are averaged
                for each release being tracked.  Hover over the pass rate
                for each release for the number of bugs reported.""",
            ),
        )

        return retval


class SmokengExtension(Extension):
    URL_PATTERNS = patterns(
        '',
        url(r'^smokeng/', include('smokeng.urls')),
    )

    LINKS = [
        {
            'name': 'smokeng',
            'url': reverse_lazy('smokeng_overview'),
            'label': 'Image',
            'index': 100,
        },
    ]

    def contribute_to_kpis(self, kpi_mgr):
        return [
            SmokengVariantPassRateKPI,
        ]

    def pull_data(self):
        call_command('jenkins_pull_smokeng')

    @classmethod
    def name(cls):
        return cls.__name__


extension = SmokengExtension()
