# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import urllib2
import re
import logging

from common.management import (
    jenkins_get,
    parse_build_description,
    JenkinsBaseCommand,
)

from common.utah_parser import UTAHParser, ParserError
from common.utils import regexes
from common.models import Bug, Artifact

from common.utils import split_build_number

from smokeng.models import (
    SmokeImage,
    SmokeResult,
    SmokeResultBug,
)
from smokeng.utah_utils import process_smokeng_log


class Command(JenkinsBaseCommand):
    job_regex = re.compile(
        ur'^(precise|quantal|raring|saucy|trusty)-' +
        ur'(desktop|server|alternate|touch(|_(ro|mir|custom|custom_demo|sf4p)))-' +
        ur'([^-_]+)(_|-smoke-)((?!static_validation).*)$'
    )

    history_days = 30

    def extract_data(self, name):
        m = self.job_regex.match(name)

        self.install_data = dict()

        if m:
            self.release = m.group(1)
            self.variant = m.group(2)
            self.arch = m.group(5)
            self.test_name = m.group(7)

            self.install_data = dict(
                release=self.release,
                variant=self.variant,
                arch=self.arch,
                test_name=self.test_name,
            )

    def process_job(self, job):
        logging.info("job.name: %s", job['name'])

    def process_build(self, build):
        logging.info("build.build_number: %s", build['number'])

        build_desc = build.get('description', '')
        build_desc_data = parse_build_description(build_desc)

        self.bugs = build_desc_data['bugs']

        image_build_number = build_desc_data['build_number']
        if image_build_number is None:
            image_build_number = self.get_build_number(build)

        if image_build_number is None:
            image_build_number = self.build_date.strftime("%Y%m%d ?")

        image, new_image = SmokeImage.objects.get_or_create(
            publish=True,
            release=self.release,
            arch=self.arch,
            variant=self.variant,
            flavor='ubuntu',
            build_number=image_build_number,
        )

        split_build_number(image)

        self.image = image

    def _process_non_utah_build(self, build):
        actions = build['actions']
        fail_count, error_count, pass_count, total_count = (0, 0, 0, 0)
        for action in actions:
            if 'failCount' in action and 'totalCount' in action:
                fail_count = action['failCount']
                total_count = action['totalCount']
                pass_count = total_count - fail_count
                break

        return dict(
            passes=pass_count,
            failures=fail_count,
            errors=error_count,
            total=total_count,
        )

    def process_artifacts(self, build, install_data):
        artifact_regex = re.compile(
            ur'^(clientlogs\/|)(.*.yaml|.*.log$|media-info|.*.props|.*.crash)'
        )
        for artifact in build['artifacts']:
            path = artifact['relativePath']
            if artifact_regex.match(path):
                logfile_path = "{}{}/artifact/{}".format(
                    self.install_data['jenkins_build'].job.url,
                    self.install_data['jenkins_build'].build_number,
                    artifact['relativePath'],
                )
                artifact, new_artifact = Artifact.objects.get_or_create(
                    publish=True,
                    url=logfile_path,
                    name=artifact['fileName'],
                    jenkins_build=install_data['jenkins_build'],
                )

                if new_artifact:
                    logging.info(
                        "Adding %s for jenkins build %s",
                        artifact.name,
                        install_data['jenkins_build'],
                    )

    def _process_logs(self, build, build_date, install_data, func):

        if not hasattr(func, '__call__'):
            raise Exception("invalid callable: {}".format(func))

        logs = self._get_utah_logs(build['artifacts'])

        for testname, log in logs:
            logfile_path = "{}{}/artifact/{}".format(
                self.install_data['jenkins_build'].job.url,
                self.install_data['jenkins_build'].build_number,
                log['relativePath'],
            )
            parser = UTAHParser()

            try:
                data = parser.parse(logfile_path)
            except ParserError as e:
                logging.warn("%s %s", logfile_path, e)
                continue

            if data is None:
                logging.warn("got null data: %s", logfile_path)
                continue

            dashboard_data = dict(
                test_name=testname,
                logfile_path=logfile_path,
                log=log,
                ran_at=data['ran_at'],
                passes=data['passes'],
                failures=data['failures'],
                errors=data['errors'],
                total=data['passes'] + data['failures'] + data['errors'],
                jenkins_build_number=build['number'],
            )

            func(dashboard_data)

        # if no utah logs are found try processing as a jenkins job
        if len(logs) == 0:
            data = self._process_non_utah_build(build)

            dashboard_data = dict(
                test_name=self.install_data['test_name'],
                ran_at=build_date,
                passes=data['passes'],
                failures=data['failures'],
                errors=data['errors'],
                total=data['passes'] + data['failures'] + data['errors'],
                jenkins_build_number=build['number'],
            )

            func(dashboard_data)

    def remove_result(self, dashboard_data):
        """ Remove smoke data from the database. """

        logging.info("Removing smoke result.")

        jenkins_builds = self.jenkins_job.jenkinsbuild_set.filter(
            build_number=dashboard_data['jenkins_build_number'],
        )

        jenkins_builds.delete()

    @staticmethod
    def _to_datetime(ran_at):
        if type(ran_at) == str:
            ran_at = datetime.datetime.strptime(ran_at, '%Y-%m-%d %H:%M:%S')
        return ran_at

    def add_result(self, dashboard_data):
        """ Add smoke data to the database. """

        logging.info("Adding smoke result.")

        result, new_result = SmokeResult.objects.get_or_create(
            publish=True,
            image=self.image,
            jenkins_build=self.install_data['jenkins_build'],
            name=dashboard_data['test_name'],
            defaults=dict(
                ran_at=dashboard_data['ran_at'],
                pass_count=dashboard_data['passes'],
                fail_count=dashboard_data['failures'],
                error_count=dashboard_data['errors'],
                total_count=dashboard_data['total'],
                status=SmokeResult.COMPLETE,
            )
        )

        if not new_result:
            result.ran_at = self._to_datetime(dashboard_data['ran_at'])
            result.pass_count = dashboard_data['passes']
            result.fail_count = dashboard_data['failures']
            result.error_count = dashboard_data['errors']
            result.total_count = dashboard_data['total']
            result.status = SmokeResult.COMPLETE
            result.save()

        all_results = SmokeResult.objects.filter(
            image=self.image,
            name=dashboard_data['test_name'],
        )
        all_results = sorted(
            all_results, key=lambda SmokeResult: int(
                SmokeResult.jenkins_build.build_number
            ),
            reverse=True,
        )

        for _result in all_results:
            _result.publish = False
            _result.save()
            logging.info(
                "Setting result to publish=False: %s",
                _result,
            )

        publish = all_results[0]
        publish.publish = True
        publish.save()

        logging.info(
            "Setting %s to publish=True",
            publish,
        )

        if 'log' in dashboard_data:
            process_smokeng_log(
                dashboard_data['log'],
                result,
                dashboard_data['logfile_path'],
            )

        self._parse_bugs(self.bugs, result)

    def _get_utah_logs(self, artifacts):
        """
        Check if there are files in 'clientlogs'.
        Returns a list of (testname, artifact) for utah logs found.
        """

        logs = []

        single = re.compile(ur'^(clientlogs\/|)utah.*.yaml')
        multi = re.compile(ur'^clientlogs\/(\S+)\/utah.*.yaml')
        for artifact in artifacts:
            path = artifact['relativePath']
            logging.debug(path)
            m = multi.match(path)
            if m:
                logs.append((m.group(1), artifact))
            elif single.match(path):
                logs.append((self.install_data['test_name'], artifact))

        return logs

    def _parse_bugs(self, lp_bugs, result):
        """
        Add bugs to the result.
        """

        for bug_no in lp_bugs:
            bug, new_bug = Bug.objects.get_or_create(
                publish=True,
                bug_no=bug_no,
            )
            self.install_data['jenkins_build'].bugs.add(bug)
            if new_bug:
                logging.info("Added new bug: %s", new_bug)
            result_bug, new_result_bug = SmokeResultBug.objects.get_or_create(
                publish=True,
                bug=bug,
                result=result,
            )
            if new_result_bug:
                logging.info("Added new result bug: %s", new_result_bug)

    def _has_media_info(self, build):
        retval = False

        for artifact in build.get('artifacts', []):
            if artifact['fileName'] == 'media_info':
                retval = True
                break

        return retval

    def get_build_number(self, build):

        retval = None

        if self._has_media_info(build):
            build_url = build['url']
            media_info_url = "{}artifact/media_info".format(build_url)
            try:
                response = jenkins_get(media_info_url, as_json=False)
            except urllib2.HTTPError:
                response = ""

            m = regexes['build-number'].match(response)

            if m:
                retval = m.group(1)
            else:
                retval = None

        return retval
