# QA Dashboard
# Copyright 2012-2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import re

from collections import Counter

from django.db import models

from common.models import (
    DashboardBaseModel,
    Bug,
    JenkinsBuild,
)

from common.utils import split_build_number


class SmokeImage(DashboardBaseModel):
    class Meta:
        db_table = 'smoke_images'

    extra_fields = [
        'summary',
    ]

    build_number = models.CharField(max_length=4096)
    build_id = models.CharField(max_length=4096, default='')
    rootfs_id = models.CharField(max_length=4096, default='')
    device_specific_id = models.CharField(max_length=4096, default='')
    customization_id = models.CharField(max_length=4096, default='')

    release = models.CharField(max_length=4096)
    flavor = models.CharField(max_length=4096)
    arch = models.CharField(max_length=4096)
    variant = models.CharField(max_length=4096)

    def save(self, **kwargs):
        '''ensure we have build_number parts'''
        if not self.build_id:
            split_build_number(self, False)
        return super(SmokeImage, self).save(**kwargs)

    def __unicode__(self):
        return "{} {} {} {}".format(
            self.release,
            self.variant,
            self.arch,
            self.build_number,
        )

    @property
    def summary(self):
        data = dict(
            pass_count=0,
            fail_count=0,
            error_count=0,
            total_count=0,
            pass_rate=0,
        )

        results = self.smokeresult_set.filter(
            publish=True,
        ).values(
            'id',
            'image__build_number',
            'image__variant',
            'image__arch',
            'image__id',
            'image__release',
            'pass_count',
            'fail_count',
            'error_count',
            'total_count',
            'name',
            'ran_at',
            'status',
            'jenkins_build__job__url',
            'jenkins_build__build_number',
        )
        crash_count = self.smokeresult_set.filter(
            publish=True,
            jenkins_build__artifact__name__endswith='.crash',
        ).values(
            'id',
            'jenkins_build__artifact__url',
        )

        bugs = SmokeResultBug.objects.filter(
            result__image=self
        ).values(
            'result__id',
            'bug__bug_no',
            'bug__title',
            'bug__status',
            'bug__assignee',
            'bug__importance',
            'bug__project',
        )

        bug_data = {}
        for bug in bugs:
            if bug['bug__bug_no'] not in bug_data:
                bug_info = dict(
                    bug_number=bug['bug__bug_no'],
                    title=bug['bug__title'],
                    assignee=bug['bug__assignee'],
                    status=bug['bug__status'],
                    importance=bug['bug__importance'],
                    project=bug['bug__project'],
                )
                bug_data[bug['bug__bug_no']] = bug_info

        # we need to give a non-queryset struct to the caller since we are
        # calculating passrates and bugs for this
        results_copy = []
        passrates = []
        for r in results:
            r['bugs'] = []
            r['crash_count'] = 0
            data['pass_count'] += r['pass_count']
            data['fail_count'] += r['fail_count']
            data['error_count'] += r['error_count']
            data['total_count'] += r['total_count']
            r['pass_rate'] = 0
            if r['total_count']:
                r['pass_rate'] = float(r['pass_count']) / r['total_count']
            r['pass_rate_pct'] = 100 * r['pass_rate']
            if r['status'] >= SmokeResult.SYNCING:
                passrates.append(r['pass_rate'])

            for b in bugs:
                if b['result__id'] == r['id']:
                    r['bugs'].append(b['bug__bug_no'])

            multi = re.compile('.*clientlogs/(.*)/.*\.crash')
            for crashes in crash_count:
                if crashes['id'] == r['id']:
                    m = multi.match(crashes['jenkins_build__artifact__url'])
                    if m:
                        if m.group(1) == r['name']:
                            # its a mega job and the test matches
                            r['crash_count'] += 1
                    else:  # non mega so show it
                        r['crash_count'] += 1

            results_copy.append(r)

        if len(passrates) == 0:
            data['pass_rate'] = 0
        else:
            data['pass_rate'] = sum(passrates) / len(passrates)
        data['pass_rate_pct'] = data['pass_rate'] * 100
        data['results'] = results_copy

        return data, bug_data

    def categories(self):
        results = self.smokeresult_set.filter(publish=True)
        results = list(SmokeTestResult.objects.filter(result__in=results)
                       .exclude(category='')
                       .values_list('category', flat=True))
        total = float(len(results))
        if total > 0.0:
            percentages = {}
            counts = Counter(results)
            for category, name in SmokeTestResult.CATEGORY_CHOICES:
                count = counts[category]
                percentages[name] = '{} ({:.0%})'.format(
                    count, float(count) / total)
            return percentages
        else:
            return None


class SmokeResult(DashboardBaseModel):
    class Meta:
        db_table = 'smoke_results'

    QUEUED = 0
    RUNNING = 1
    SYNCING = 2
    COMPLETE = 3

    STATUS_CHOICES = (
        (QUEUED, 'Queued'),
        (RUNNING, 'Running'),
        (SYNCING, 'Syncing'),
        (COMPLETE, 'Complete'),
    )

    image = models.ForeignKey(SmokeImage)
    jenkins_build = models.ForeignKey(JenkinsBuild)
    name = models.CharField(max_length=4096)
    fail_count = models.IntegerField()
    error_count = models.IntegerField(null=False, default=0)
    pass_count = models.IntegerField()
    total_count = models.IntegerField()
    ran_at = models.DateTimeField('date run')
    status = models.IntegerField(choices=STATUS_CHOICES, default=COMPLETE)
    pass_rate = models.FloatField(default=0.0)

    def save(self, **kwargs):
        '''ensure we store the pass_rate'''
        self.pass_rate = 0
        if self.total_count > 0:
            self.pass_rate = float(self.pass_count) / self.total_count
        return super(SmokeResult, self).save(**kwargs)

    @staticmethod
    def create_queued_results(image, build, tests):
        ran_at = datetime.datetime.now()
        for test in tests:
            result, created = SmokeResult.objects.get_or_create(
                image=image,
                name=test,
                defaults={
                    'fail_count': 0,
                    'error_count': 0,
                    'pass_count': 0,
                    'total_count': 0,
                    'ran_at': ran_at,
                    'jenkins_build': build,
                    'status': SmokeResult.QUEUED,
                },
            )
            if not created:
                result.status = SmokeResult.QUEUED
                result.ran_at = ran_at
                result.jenkins_build = build
                result.save()

    @staticmethod
    def set_running(image, build, tests):
        for test in tests:
            result = SmokeResult.objects.get(
                image=image,
                jenkins_build=build,
                name=test,
            )
            result.ran_at = datetime.datetime.now()
            result.status = SmokeResult.RUNNING
            result.save()

    @staticmethod
    def set_syncing(image, build, test, passes, fails, errors):
        result = SmokeResult.objects.get(
            image=image,
            jenkins_build=build,
            name=test,
        )
        result.status = SmokeResult.SYNCING
        result.pass_count = passes
        result.fail_count = fails
        result.error_count = errors
        result.total_count = passes + fails + errors
        result.save()

    def categories(self):
        results = list(self.smoketestresult_set.exclude(category='')
                       .values_list('category', flat=True))
        total = float(len(results))
        if total > 0.0:
            percentages = {}
            counts = Counter(results)
            for category, name in SmokeTestResult.CATEGORY_CHOICES:
                count = counts[category]
                percentages[name] = '{} ({:.0%})'.format(
                    count, float(count) / total)
            return percentages
        else:
            return None

    def __unicode__(self):
        return "{} - {}".format(
            self.name,
            self.ran_at,
        )


class SmokeResultBug(DashboardBaseModel):
    class Meta:
        db_table = "smoke_result_bugs"

    bug = models.ForeignKey(Bug)
    result = models.ForeignKey(SmokeResult)


class SmokeTestResult(DashboardBaseModel):
    class Meta:
        db_table = 'smoke_tests'

    UNCATEGORIZED = 'U'
    APPLICATION_REGRESSION = 'R'
    TEST_PROBLEM = 'T'
    INFRASTRUCTURE_ISSUE = 'I'
    AUTOPILOT_BUG = 'A'

    CATEGORY_CHOICES = (
        (UNCATEGORIZED, 'Uncategorized'),
        (APPLICATION_REGRESSION, 'Application Regression'),
        (TEST_PROBLEM, 'Test Problem'),
        (INFRASTRUCTURE_ISSUE, 'Infrastructure Issue'),
        (AUTOPILOT_BUG, 'Autopilot Bug'),
    )

    result = models.ForeignKey(SmokeResult)
    command_type = models.CharField(max_length=4096)
    command = models.CharField(max_length=4096)
    testcase = models.CharField(max_length=4096)
    testsuite = models.CharField(max_length=4096)
    returncode = models.IntegerField()
    stderr = models.TextField(null=True)
    stdout = models.TextField(null=True)
    category = models.CharField(
        max_length=1,
        choices=CATEGORY_CHOICES,
        verbose_name='Cause',
        blank=True,
    )
