# QA Dashboard
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.utils.http import urlquote

import django_tables2 as tables
from django_tables2.utils import Accessor as A


class LinkColumnQuoted(tables.LinkColumn):

    def render_link(self, uri, text, attrs=None):
        uri = urlquote(uri, safe='/: %')
        return super(LinkColumnQuoted, self).render_link(uri, text, attrs)


class SmokeBaseTable(tables.Table):
    class Meta:
        attrs = {'class': 'basic wide'}
        fields = (
            'total_count',
            'pass_count',
            'fail_count',
            'error_count',
            'crash_count',
            'pass_rate',
            'bugs',
        )

    total_count = tables.Column(
        verbose_name='Total',
        attrs={'td': {'class': 'num'}},
    )
    pass_count = tables.Column(
        verbose_name='Pass',
        attrs={'td': {'class': 'num'}},
    )
    fail_count = tables.Column(
        verbose_name='Fail',
        attrs={'td': {'class': 'num'}},
    )
    error_count = tables.TemplateColumn(
        '{% if record.error_count > 0 and record.bugs|length = 0 %}'
        '<div class="error-14x14 smaller-icon icon"></div>{% endif %}'
        '{{ record.error_count }}',
        verbose_name='error',
        attrs={'td': {'class': 'num'}},
    )
    crash_count = tables.Column(
        verbose_name='Crashes',
        attrs={'td': {'class': 'num'}},
    )
    pass_rate = tables.TemplateColumn(
        '''{% load dashboard_extras %}
        {% if record.inprogress %}
          Running
        {% else %}
        {{ record.pass_rate|decimal_to_percent }}
        {% endif %}''',
        attrs={'td': {'class': 'num pass_rate_color'}},
    )
    td_class_extra = tables.TemplateColumn(
        '{% load dashboard_extras %}'
        '{{ record.pass_rate|pass_rate_color }}',
        visible=False,
    )
    bugs = tables.TemplateColumn(
        template_name='bug.html',
    )


class SmokeImageTable(SmokeBaseTable):
    class Meta(SmokeBaseTable.Meta):
        order_by = ('-build_number',)
        fields = (
            'build_number',
            'variant',
            'arch',
        ) + SmokeBaseTable.Meta.fields

    build_number = LinkColumnQuoted(
        'smokeng.views.image_overview',
        accessor="build_number",
        order_by=(
            'rootfs_id',
            'build_id',
            'device_specific_id',
            'customization_id',
            'build_number',
        ),
        kwargs={
            "release": A('release'),
            "variant": A('variant'),
            "arch": A('arch'),
            "build_number": A('build_number'),
            "image_id": A('image_id'),
        }
    )
    variant = tables.Column()
    arch = tables.Column()


class SmokeResultTable(SmokeBaseTable):
    class Meta(SmokeBaseTable.Meta):
        order_by = ('-ran_at', 'name',)
        fields = (
            'name',
            'ran_at',
        ) + SmokeBaseTable.Meta.fields

    name = LinkColumnQuoted(
        "smokeng.views.image_test_results",
        kwargs={
            "release": A('image__release'),
            "variant": A('image__variant'),
            "arch": A('image__arch'),
            "build_number": A('image__build_number'),
            "image_id": A('image__id'),
            "name": A('name'),
        },
    )
    ran_at = tables.TemplateColumn(
        '{% load dashboard_extras %}'
        '{{ record|smokeresult_ran_at }}'
    )


class SmokeKPITable(tables.Table):
    class Meta:
        order_by = ('arch')

    arch = tables.Column()
    build_number = tables.Column(
        verbose_name="Build",
    )
    pass_rate = tables.TemplateColumn(
        '{{ record.pass_rate|floatformat }}%',
        attrs={'td': {'class': 'num'}},
    )


class SmokeTestResultTable(tables.Table):
    command_type = LinkColumnQuoted(
        "smokeng.views.test_result_detail",
        kwargs={
            "release": A('release'),
            "variant": A('variant'),
            "arch": A('arch'),
            "build_number": A('build_number'),
            "image_id": A('image_id'),
            "name": A('name'),
            "id": A('id'),
        },
    )
    testcase = tables.Column()
    returncode = tables.TemplateColumn(
        '{{ record.returncode }}',
        verbose_name='Return code',
        attrs={'td': {'class': 'num pass_rate_color'}},
    )
    id = tables.Column(visible=False)
    td_class_extra = tables.TemplateColumn(
        '{% load dashboard_extras %}'
        '{{ record.returncode|zero_color }}',
        visible=False,
    )

    class Meta:
        attrs = {'class': 'basic wide'}
        order_by = ('id', )
