# QA Dashboard
# Copyright 2012-2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import json
import mock
import urllib2

# this is required to pull in the tastypie hack at the bottom of
# smokeng/api.py. Without this, our tests won't pass
import smokeng.api
smokeng.api  # just to silence pep8 warnings. this does nothing

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client
from django.contrib.auth.models import (
    Group,
    User,
)

from tastypie.models import ApiKey
from tastypie.test import TestApiClient

from smokeng.models import (
    SmokeImage,
    SmokeResult,
    SmokeResultBug,
    SmokeTestResult,
)

from smokeng.management.commands.jenkins_pull_smokeng import Command
from smokeng.config import get_internal_jenkins, primary_targets
from smokeng import views

from common.models import (
    Bug,
    JenkinsBuild,
    JenkinsJob,
    UserProfile,
    Artifact
)


def _setUp(self):
    """ Basic setup for each test. """

    self.jenkins_job = JenkinsJob.objects.create(
        name="blah-blah-blah",
        url=(
            'http://jenkins.qa.ubuntu.com/job/saucy-touch_mir-mako-smoke-'
            'notes-app-autopilot/'
        ),
    )
    self.assert_(self.jenkins_job)

    self.jenkins_build = JenkinsBuild.objects.create(
        job=self.jenkins_job,
        build_number=1,
        ran_at=datetime.datetime.now(),
    )
    self.assert_(self.jenkins_build)

    self.image = SmokeImage.objects.create(
        release="saucy",
        arch="mako",
        variant="touch_mir",
        build_number="20130401",
    )
    self.assert_(self.image)

    self.result = SmokeResult.objects.create(
        image=self.image,
        jenkins_build=self.jenkins_build,
        name="notes-app-autopilot",
        fail_count=1,
        error_count=1,
        pass_count=1,
        total_count=3,
        ran_at=datetime.datetime.now(),
    )
    self.assert_(self.result)

    self.bug = Bug.objects.create(
        bug_no=1,
        status='New',
        importance='Low',
        assignee='Chris Johnston',
    )
    self.assert_(self.bug)

    self.result_bug = SmokeResultBug.objects.create(
        bug=self.bug,
        result=self.result,
    )
    self.assert_(self.result_bug)

    self.testcaseresult = SmokeTestResult.objects.create(
        result=self.result,
        command_type="testcase_test",
        command="./test.py",
        testcase="test-testcase",
        testsuite="touch",
        returncode="0",
    )


class TestSmokeURLS(TestCase):

    def setUp(self):
        self.c = Client()

        _setUp(self)

    def _test_url(self, url):
        response = self.c.get(url)

        self.assertEqual(response.status_code, 200)

    def test_release(self):

        url = reverse(
            'smokeng_overview',
            args=(
                self.image.release,
            ),
        )

        self._test_url(url)

    def test_release_bug_sorting(self):

        url = reverse(
            'smokeng_overview',
            args=(
                self.image.release,
            ),
        )

        self._test_url("{}?sort=bugs".format(url))

    def test_image_overview_url(self):

        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )
        self._test_url(url)

    def test_image_overview_touch_ro_url(self):
        self.image.build_number = '56:20130920:20130920'
        self.image.save()
        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )
        self._test_url(url)

    def test_image_overview_unknown_build_number_url(self):
        self.image.build_number = '20130920 ?'
        self.image.save()
        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )

        # URL quote the url due to the '?'
        url = urllib2.quote(url)
        self._test_url(url)

    def test_image_bug_sorting(self):

        url = reverse(
            'smokeng_image_overview',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
            ),
        )

        self._test_url("{}?sort=bugs".format(url))

    def test_image_test_results_url(self):
        url = reverse(
            'smokeng_image_test_results',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
            ),
        )
        response = self._test_url(url)
        self.assertTemplateUsed(response, 'smokeng/test_results.html')

    def test_image_test_results_touch_ro_url(self):
        self.image.build_number = '56:20130920:20130920'
        self.image.save()
        url = reverse(
            'smokeng_image_test_results',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
            ),
        )
        self._test_url(url)

    def test_image_test_results_unknown_build_number_url(self):
        self.image.build_number = '20130920 ?'
        self.image.save()
        url = reverse(
            'smokeng_image_test_results',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
            ),
        )

        # URL quote the url due to the '?'
        url = urllib2.quote(url)
        self._test_url(url)

    def test_test_result_detail_url(self):
        url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )
        response = self._test_url(url)
        self.assertTemplateUsed(response, 'smokeng/test_result_detail.html')

    def test_image_test_result_detail_touch_ro_url(self):
        self.image.build_number = '56:20130920:20130920'
        self.image.save()
        url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )
        self._test_url(url)

    def test_image_test_result_detail_unknown_build_number_url(self):
        self.image.build_number = '20130920 ?'
        self.image.save()
        url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )

        # URL quote the url due to the '?'
        url = urllib2.quote(url)
        self._test_url(url)


class TestSmoke(TestCase):
    """ Tests for the smoke models. """

    def setUp(self):
        _setUp(self)

    def test_image_aggregations_pass(self):
        """ Test aggregated pass values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['pass_count'], 1)

    def test_image_aggregations_fail(self):
        """ Test aggregated fail values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['fail_count'], 1)

    def test_image_aggregations_error(self):
        """ Test aggregated error values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['error_count'], 1)

    def test_image_aggregations_total(self):
        """ Test aggregated total values for an image instance. """
        summary, bug_data = self.image.summary

        self.assertEqual(summary['total_count'], 3)

    def test_image_aggregations_pass_rate(self):
        """ Test aggregated pass_rate values for an image instance. """
        summary, bug_data = self.image.summary

        # cast to int to make comparison more reliable
        self.assertEqual(int(summary['pass_rate'] * 100), 33)

    def test_result_bugs(self):
        """ Test bug list for a result instance. """

        self.assertEqual(self.result.smokeresultbug_set.count(), 1)

    def test_crash_count_empty(self):
        results = views._results_overview(SmokeResult.objects.all())
        totals = views.get_totals(results, 'saucy')[0].values()
        self.assertEqual(0, totals[0]['crash_count'])

    def test_crash_count_simple(self):
        Artifact.objects.create(
            url='http://foo/clientlogs/notes-app-autopilot/test.crash',
            name='test.crash',
            jenkins_build=self.jenkins_build,
        )
        results = views._results_overview(SmokeResult.objects.all())
        totals = views.get_totals(results, 'saucy')[0].values()
        self.assertEqual(1, totals[0]['crash_count'])

    def test_crash_count_complex(self):
        Artifact.objects.create(
            url='http://foo/clientlogs/notes-app-autopilot/test.crash',
            name='test.crash',
            jenkins_build=self.jenkins_build,
        )
        Artifact.objects.create(
            url='http://foo/clientlogs/notes-app-autopilot/test-other.crash',
            name='test-other.crash',
            jenkins_build=self.jenkins_build,
        )

        results = views._results_overview(SmokeResult.objects.all())
        totals = views.get_totals(results, 'saucy')[0].values()
        self.assertEqual(2, totals[0]['crash_count'])

        # now create a new run that has a different number of failures
        # NOTE: our pull script and REST API reuse the existing SmokeResult
        rerun_build = JenkinsBuild.objects.create(
            job=self.jenkins_job,
            build_number=2,
            ran_at=datetime.datetime.now(),
        )

        self.result.jenkins_build = rerun_build
        self.result.save()

        results = views._results_overview(SmokeResult.objects.all())
        totals = views.get_totals(results, 'saucy')[0].values()
        self.assertEqual(0, totals[0]['crash_count'])

        # now add a failure to the re-run
        Artifact.objects.create(
            url='http://foo/clientlogs/notes-app-autopilot/test-other.crash',
            name='test-other.crash',
            jenkins_build=rerun_build,
        )
        results = views._results_overview(SmokeResult.objects.all())
        totals = views.get_totals(results, 'saucy')[0].values()
        self.assertEqual(1, totals[0]['crash_count'])


class TestSmokeMethods(TestCase):

    def setUp(self):
        self.command = Command()
        self.build = dict(
            url="http://blah",
            artifacts=[
                dict(
                    fileName="media_info",
                ),
            ],
        )

    def test_get_build_number_none(self):
        build = {}

        self.assertIsNone(self.command.get_build_number(build))

    @mock.patch('urllib2.Request')
    @mock.patch('urllib2.urlopen')
    def test_get_build_number_media_info(self, urlopen, request):
        build_number = '20111012'
        url_resp = 'Ubuntu 11.10 "Oneiric Ocelot" - Release amd64 ({})'.format(
            build_number,
        )
        contents = mock.Mock()
        contents.read.return_value = url_resp
        urlopen.return_value = contents

        self.assertEqual(
            self.command.get_build_number(self.build),
            build_number,
        )

    def test__has_media_info_true(self):

        self.assertTrue(self.command._has_media_info(self.build))

    def test__has_media_info_false(self):

        self.build = {}

        self.assertFalse(self.command._has_media_info(self.build))


class TestPullScript(TestCase):
    def testJobNames(self):
        '''ensure the job_regex matches everything it should.'''
        good = [
            'trusty-touch-mako-smoke-dropping-letters-app-autopilot',
            'saucy-touch_ro-maguro-smoke-master',
            'precise-server-amd64-smoke-default',
            'trusty-desktop-i386-smoke-default',
        ]
        bad = [
            'foo-touch_ro-maguro-smoke-master',  # no such release
            'trusty-touch_ro-maguro-smoke',      # no test defined
            'trusty-desktop-i386-smoke-static_validation',
        ]

        for x in good:
            self.assertIsNotNone(Command.job_regex.match(x))
        for x in bad:
            self.assertIsNone(Command.job_regex.match(x))

    def _extract_matches(self, name, expected):
        c = Command()
        c.extract_data(name)
        self.assertDictEqual(expected, c.install_data)

    def testExtractData(self):
        '''ensure the extract data can populate from the job name.'''
        patterns = [
            ('saucy-touch_ro-maguro-smoke-master', {
                'arch': 'maguro',
                'release': 'saucy',
                'test_name': 'master',
                'variant': 'touch_ro',
            }),
            ('trusty-touch-mako-smoke-friends-app-autopilot', {
                'arch': 'mako',
                'release': 'trusty',
                'test_name': 'friends-app-autopilot',
                'variant': 'touch',
            }),
        ]
        for name, data in patterns:
            self._extract_matches(name, data)

    def testUtahLogs(self):
        '''ensure we find the proper patterns of logfile names'''
        good = [
            {'relativePath': 'utah.yaml'},
            {'relativePath': 'clientlogs/utah.yaml'},
        ]
        good_multi = [
            {'relativePath': 'clientlogs/test1/utah.yaml'},
            {'relativePath': 'clientlogs/test2/utah.yaml'},
        ]
        bad = [
            {'relativePath': 'foo/utah.yaml'},
            {'relativePath': 'clientlogs/utah-host.yml'},
        ]
        c = Command()
        c.install_data = {'test_name': 'foo'}
        logs = c._get_utah_logs(good + good_multi + bad)
        expected = [(c.install_data['test_name'], x) for x in good]
        for x in good_multi:
            expected.append((x['relativePath'].split('/')[1], x))
        self.assertListEqual(expected, logs)


class TestSmokeConfig(TestCase):

    def setUp(self):
        self.jenkins = 'http://d-jenkins.ubuntu-ci:8080/'
        self.touch_jenkins = 'http://q-jenkins.ubuntu-ci:8080/'
        self.primary_touch = ('mako', 'flo', 'manta')
        self.primary_touchro = ('mako', 'maguro')

    def test_get_touch_internal_url(self):
        variant = 'touch'
        internal = get_internal_jenkins(variant)
        self.assertEquals(internal, self.touch_jenkins)

    def test_get_touch_ro_internal_url(self):
        variant = 'touch_ro'
        internal = get_internal_jenkins(variant)
        self.assertEquals(internal, self.touch_jenkins)

    def test_get_desktop_internal_url(self):
        variant = 'desktop'
        internal = get_internal_jenkins(variant)
        self.assertEquals(internal, self.jenkins)

    def test_get_server_internal_url(self):
        variant = 'server'
        internal = get_internal_jenkins(variant)
        self.assertEquals(internal, self.jenkins)

    def test_primary_targets_touch(self):
        variant = 'touch'
        targets = primary_targets(variant)
        self.assertEquals(targets, self.primary_touch)

    def test_primary_targets_touch_ro(self):
        variant = 'touch_ro'
        targets = primary_targets(variant)
        self.assertEquals(targets, self.primary_touchro)

    def test_primary_targets_server(self):
        variant = 'server'
        targets = primary_targets(variant)
        self.assertEquals(targets, None)

    def test_primary_targets_desktop(self):
        variant = 'desktop'
        targets = primary_targets(variant)
        self.assertEquals(targets, None)


class TestInternalJenkinsURLSupport(TestCase):
    '''
    Test the support for giving users private URLs if the user elects to have
    private URLs, otherwise show public URLs
    '''

    def setUp(self):
        self.c = Client()
        self.public_jenkins_build = (
            '<td><strong>Build #:</strong></td><td><a href="http://jenkins.'
            'qa.ubuntu.com/job/saucy-touch_mir-mako-smoke-notes-app-'
            'autopilot/1">20130401</a></td>'
        )
        self.private_jenkins_build = (
            '<td><strong>Build #:</strong></td><td><a href="http://'
            'q-jenkins.ubuntu-ci:8080/job/'
            'saucy-touch_mir-mako-smoke-notes-app-autopilot/1">20130401</a>'
            '</td>'
        )
        self.public_jenkins_artifact = (
            '<li><a href="http://jenkins.qa.ubuntu.com/job/saucy-touch'
            '_mir-mako-smoke-notes-app-autopilot/1/artifact/clientlogs/'
            'utah.yaml/*view*/">utah.yaml</a></li>'
        )
        self.private_jenkins_artifact = (
            '<li><a href="http://q-jenkins.ubuntu-ci:8080/job/'
            'saucy-touch_mir-mako-smoke-notes-app-autopilot/1/artifact/'
            'clientlogs/utah.yaml/*view*/">utah.yaml</a></li>'
        )
        _setUp(self)

        self.artifact = Artifact.objects.create(
            url=(
                'http://jenkins.qa.ubuntu.com/job/saucy-touch_mir-mako'
                '-smoke-notes-app-autopilot/1/artifact/clientlogs/utah.yaml'
            ),
            name='utah.yaml',
            jenkins_build=self.jenkins_build,
        )

        self.user = User.objects.create(
            username='testuser',
            first_name='Test',
            last_name='User',
        )
        self.user.set_password('password')
        self.user.save()
        self.assert_(self.user)

    def tearDown(self):
        self.c.logout()

    def login(self):
        logged_in = self.c.login(
            username='testuser',
            password='password',
        )
        self.assertTrue(logged_in)

    def _create_profile(self, private=False):
        UserProfile.objects.create(
            user=self.user,
            use_private_url=private,
        )

    def _test_url(self, url):
        response = self.c.get(url)
        self.assertEqual(response.status_code, 200)
        return response

    def _image_test_results(self):
        url = reverse(
            'smokeng_image_test_results',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
            ),
        )
        response = self._test_url(url)
        return response

    def _test_result_detail(self):
        url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )
        response = self._test_url(url)
        return response

    def test_not_logged_in_public_url_test_case_results(self):
        '''
        Test that a user who is not logged in receives the public Jenkins URLs
        for the "Test Case Results" page
        '''
        page = self._image_test_results()
        self.assertContains(page, self.public_jenkins_build)
        self.assertContains(page, self.public_jenkins_artifact)

    def test_not_logged_in_public_url_test_case(self):
        '''
        Test that a user who is not logged in receives the public Jenkins URLs
        for the "Test Case" page
        '''
        page = self._test_result_detail()
        self.assertContains(page, self.public_jenkins_build)
        self.assertContains(page, self.public_jenkins_artifact)

    def test_logged_in_no_profile_public_url_test_case_results(self):
        '''
        Test that a user who is logged in but does not have a profile receives
        the public Jenkins URLs for the "Test Case Results" page
        '''
        self.login()
        page = self._image_test_results()
        self.assertContains(page, self.public_jenkins_build)
        self.assertContains(page, self.public_jenkins_artifact)

    def test_logged_in_no_profile_public_url_test_case(self):
        '''
        Test that a user who is logged in but does not have a profile receives
        the public Jenkins URLs for the "Test Case" page
        '''
        self.login()
        page = self._test_result_detail()
        self.assertContains(page, self.public_jenkins_build)
        self.assertContains(page, self.public_jenkins_artifact)

    def test_logged_in_profile_external_public_url_test_case_results(self):
        '''
        Test that a user who is logged in with a profile but has not selected
        to see internal URLs receives the public Jenkins URLs for the "Test
        Case Results" page
        '''
        self._create_profile()
        self.login()
        page = self._image_test_results()
        self.assertContains(page, self.public_jenkins_build)
        self.assertContains(page, self.public_jenkins_artifact)

    def test_logged_in_profile_external_public_url_test_case(self):
        '''
        Test that a user who is logged in with a profile but has not selected
        to see internal URLs receives the public Jenkins URLs for the "Test
        Case" page
        '''
        self._create_profile()
        self.login()
        page = self._test_result_detail()
        self.assertContains(page, self.public_jenkins_build)
        self.assertContains(page, self.public_jenkins_artifact)

    def test_logged_in_profile_internal_private_url_test_case_results(self):
        '''
        Test that a user who is logged in with a profile and has selected
        to see internal URLs receives the private Jenkins URLs for the "Test
        Case Results" page
        '''
        self._create_profile(private=True)
        self.login()
        page = self._image_test_results()
        self.assertContains(page, self.private_jenkins_build)
        self.assertContains(page, self.private_jenkins_artifact)

    def test_logged_in_profile_internal_private_url_test_case(self):
        '''
        Test that a user who is logged in with a profile and has selected
        to see internal URLs receives the private Jenkins URLs for the "Test
        Case" page
        '''
        self._create_profile(private=True)
        self.login()
        page = self._test_result_detail()
        self.assertContains(page, self.private_jenkins_build)
        self.assertContains(page, self.private_jenkins_artifact)


class TestSmokeStatus(TestCase):
    """ Tests for live status operations """

    def setUp(self):
        _setUp(self)

        # make a basic assumption on what's already in the DB for
        # tests below
        self.assertEqual(1, SmokeResult.objects.all().count())

        self.client = Client()

    def _get_results_url(self):
        return '/smokeng/%s/%s/%s/%s/%d/' % (
            self.image.release,
            self.image.variant,
            self.image.arch,
            self.image.build_number,
            self.image.id,
        )

    def testQueued(self):
        tests = ['t1', 't2', 't3']
        SmokeResult.create_queued_results(
            self.image, self.jenkins_build, tests)

        qs = SmokeResult.objects.filter(status=SmokeResult.QUEUED)
        self.assertEqual(len(tests), qs.count())

        # ensure pass-rates aren't affected
        summary = self.image.summary
        self.assertEqual(3, summary[0]['total_count'])
        self.assertEqual(1.0 / 3.0, summary[0]['pass_rate'])

        resp = self.client.get('/smokeng/')
        pass_rate = resp.context[-1]['table'].rows[0]['pass_rate']
        self.assertEqual(u'Running', pass_rate.strip())

        resp = self.client.get(self._get_results_url())
        rows = resp.context[-1]['table'].rows
        for i in range(len(rows)):
            if i < len(rows) - 1:
                self.assertEqual('Queued', rows[i]['ran_at'])
            else:
                self.assertNotEqual('Queued', rows[i]['ran_at'])

    def testRunning(self):
        tests = ['t1', 't2', 't3']
        SmokeResult.create_queued_results(
            self.image, self.jenkins_build, tests)

        qs = SmokeResult.objects.filter(status=SmokeResult.QUEUED)
        self.assertEqual(len(tests), qs.count())
        ran_at = qs[0].ran_at

        # put the first few into running and make sure the counts add up
        SmokeResult.set_running(self.image, self.jenkins_build, tests[:-1])
        self.assertEqual(len(tests) + 1, SmokeResult.objects.all().count())
        qs = SmokeResult.objects.filter(status=SmokeResult.QUEUED)
        self.assertEqual(1, qs.count())
        self.assertEqual(ran_at, qs[0].ran_at)
        qs = SmokeResult.objects.filter(status=SmokeResult.RUNNING)
        self.assertEqual(len(tests) - 1, qs.count())

        # put the last result into running and make sure the counts add up
        SmokeResult.set_running(self.image, self.jenkins_build, tests[-1:])
        self.assertEqual(len(tests) + 1, SmokeResult.objects.all().count())
        qs = SmokeResult.objects.filter(status=SmokeResult.QUEUED)
        self.assertEqual(0, qs.count())
        qs = SmokeResult.objects.filter(status=SmokeResult.RUNNING)
        self.assertEqual(len(tests), qs.count())
        self.assertGreater(qs[0].ran_at, ran_at)

        # ensure pass-rates aren't affected
        self.assertEqual(3, self.image.summary[0]['total_count'])
        self.assertEqual(1.0 / 3.0, self.image.summary[0]['pass_rate'])

    def testSyncing(self):
        tests = ['t1', 't2', 't3']
        SmokeResult.create_queued_results(
            self.image, self.jenkins_build, tests)

        qs = SmokeResult.objects.filter(status=SmokeResult.QUEUED)
        self.assertEqual(len(tests), qs.count())

        # put tests into a few states to ensure it can handle them all
        SmokeResult.set_running(self.image, self.jenkins_build, tests[:-1])

        # this will have wind up with 4 results:
        # the original from _setup with a pass-rate 33%
        # t1 = which will be 0, t2 and t3 which will be 33%
        i = 0
        for t in tests:
            SmokeResult.set_syncing(self.image, self.jenkins_build, t, i, i, i)
            i += 1
        qs = SmokeResult.objects.filter(status=SmokeResult.SYNCING)
        self.assertEqual(len(tests), qs.count())
        self.assertEqual(12, self.image.summary[0]['total_count'])
        self.assertEqual(.25, self.image.summary[0]['pass_rate'])

        resp = self.client.get('/smokeng/')
        pass_rate = resp.context[-1]['table'].rows[0]['pass_rate']
        self.assertEqual(u'25%', pass_rate.strip())

        resp = self.client.get(self._get_results_url())
        rows = resp.context[-1]['table'].rows
        for i in range(len(rows)):
            if i < len(rows) - 1:
                self.assertEqual('Syncing', rows[i]['ran_at'])
            else:
                self.assertNotEqual('Syncing', rows[i]['ran_at'])

    def testRepeat(self):
        '''Ensure we can move a result back to QUEUED properly.'''
        tests = ['t1', 't2', 't3']
        SmokeResult.create_queued_results(
            self.image, self.jenkins_build, tests)

        # put tests into a few states to ensure it can handle them all
        # the syncing result bumps up the "total count" by 3 to 6
        SmokeResult.set_running(self.image, self.jenkins_build, tests[:-1])
        SmokeResult.set_syncing(self.image, self.jenkins_build,
                                tests[-1], 1, 1, 1)

        SmokeResult.create_queued_results(
            self.image, self.jenkins_build, tests)

        # ensure we haven't created new rows:
        self.assertEqual(len(tests) + 1, SmokeResult.objects.all().count())

        qs = SmokeResult.objects.filter(status=SmokeResult.QUEUED)
        self.assertEqual(len(tests), qs.count())

        # ensure pass-rates go back properly
        summary = self.image.summary
        self.assertEqual(6, summary[0]['total_count'])
        self.assertEqual(1.0 / 3.0, summary[0]['pass_rate'])

        resp = self.client.get('/smokeng/')
        pass_rate = resp.context[-1]['table'].rows[0]['pass_rate']
        self.assertEqual(u'Running', pass_rate.strip())


class TestSmokeApi(TestCase):

    def setUp(self):
        _setUp(self)

        # create an api key for "post" operations
        u = User()
        u.username = 'admin'
        u.is_superuser = True
        u.save()

        k = ApiKey()
        k.user = u
        k.key = 'key'
        k.save()
        self.user = u.username
        self.key = k.key

        # make a basic assumption on what's already in the DB for
        # tests below
        self.assertEqual(1, SmokeResult.objects.all().count())
        self.client = TestApiClient()

    def _get(self, resource, params=None):
        return json.loads(self.client.get(resource, data=params).content)

    def _post(self, resource, params):
        auth = 'ApiKey %s:%s' % (self.user, self.key)
        resp = self.client.post(resource, data=params, authentication=auth)
        self.assertEqual(201, resp.status_code)
        return resp['location']

    def _patch(self, resource, params):
        auth = 'ApiKey %s:%s' % (self.user, self.key)
        resp = self.client.patch(resource, data=params, authentication=auth)
        self.assertEqual(202, resp.status_code)

    def testBuildGet(self):
        params = {
            'build_number': self.jenkins_build.build_number,
            'job': self.jenkins_job.id,
        }
        obj = self._get('/smokeng/api/v1/build/', params)
        self.assertEqual(1, len(obj['objects']))
        obj = obj['objects'][0]
        self.assertEqual(self.jenkins_build.ran_at.isoformat(), obj['ran_at'])

    def testImageGet(self):
        params = {
            'build_number': self.image.build_number,
            'release': self.image.release,
            'variant': self.image.variant,
            'arch': self.image.arch,
        }
        obj = self._get('/smokeng/api/v1/image/', params)
        self.assertEqual(1, len(obj['objects']))
        obj = obj['objects'][0]
        self.assertEqual(self.image.build_number, obj['build_number'])
        self.assertEqual(self.image.release, obj['release'])

    def testResultGet(self):
        params = {
            'jenkins_build': self.jenkins_build.id,
            'image': self.image.id,
            'name': self.result.name,
        }
        obj = self._get('/smokeng/api/v1/result/', params)
        self.assertEqual(1, len(obj['objects']))
        obj = obj['objects'][0]
        self.assertEqual(self.result.name, obj['name'])
        self.assertEqual(self.result.total_count, obj['total_count'])
        self.assertEqual(self.result.pass_count, obj['pass_count'])
        self.assertEqual(self.result.status, obj['status'])

    def testBuildAdd(self):
        params = {
            'build_number': 'build_number',
            'job': '/smokeng/api/v1/job/%d/' % self.jenkins_job.id,
            'ran_at': datetime.datetime.now().isoformat(),
            'build_description': 'inprogress',
        }
        loc = self._post('/smokeng/api/v1/build/', params)
        obj = self._get(loc)
        self.assertEqual(obj['build_number'], params['build_number'])
        self.assertEqual(obj['build_description'], params['build_description'])
        self.assertEqual(obj['job'], params['job'])
        self.assertEqual(obj['ran_at'], params['ran_at'])

    def testImageAdd(self):
        params = {
            'build_number': 'build_number',
            'release': 'release',
            'flavor': 'flavor',
            'variant': 'variant',
            'arch': 'arch',
        }
        loc = self._post('/smokeng/api/v1/image/', params)
        obj = self._get(loc)
        self.assertEqual(params['arch'], obj['arch'])
        self.assertEqual(params['release'], obj['release'])
        self.assertEqual(params['build_number'], obj['rootfs_id'])

    def testResultAdd(self):
        params = {
            'ran_at': datetime.datetime.now().isoformat(),
            'status': 0,
            'total_count': 6,
            'pass_count': 3,
            'error_count': 2,
            'fail_count': 1,
            'jenkins_build': ('/smokeng/api/v1/build/%d/' %
                              self.jenkins_build.id),
            'image': '/smokeng/api/v1/image/%d/' % self.image.id,
            'name': 'the-test-result-name',
        }
        loc = self._post('/smokeng/api/v1/result/', params)
        obj = self._get(loc)
        self.assertEqual(params['name'], obj['name'])
        self.assertEqual(params['ran_at'], obj['ran_at'])
        self.assertEqual(params['status'], obj['status'])
        self.assertEqual(params['total_count'], obj['total_count'])
        self.assertEqual(params['pass_count'], obj['pass_count'])
        self.assertEqual(params['error_count'], obj['error_count'])
        self.assertEqual(params['fail_count'], obj['fail_count'])
        self.assertEqual(params['jenkins_build'], obj['jenkins_build'])
        self.assertEqual(params['image'], obj['image'])

    def testResultAddWithResults(self):
        '''a smokeresult with utah results included'''
        params = {
            'ran_at': datetime.datetime.now().isoformat(),
            'status': 0,
            'results': {
                'passes': 2,
                'errors': 1,
                'failures': 1,
                'commands': [
                    # include one smoketestresult to ensure it can be processed
                    {
                        'command': 'foo-cmd',
                        'cmd_type': 'foo-type',
                        'testcase': 'foo-testcase',
                        'testsuite': 'foo-testsuite',
                        'stdout': 'foo-stdout',
                        'stderr': 'foo-stderr',
                        'returncode': 0,
                    },
                ]
            },
            'jenkins_build': ('/smokeng/api/v1/build/%d/' %
                              self.jenkins_build.id),
            'image': '/smokeng/api/v1/image/%d/' % self.image.id,
            'name': 'the-test-result-name',
        }
        loc = self._post('/smokeng/api/v1/result/', params)
        obj = self._get(loc)
        self.assertEqual(params['name'], obj['name'])
        self.assertEqual(params['ran_at'], obj['ran_at'])
        self.assertEqual(params['status'], obj['status'])
        self.assertEqual(4, obj['total_count'])
        self.assertEqual(2, obj['pass_count'])
        self.assertEqual(1, obj['error_count'])
        self.assertEqual(1, obj['fail_count'])
        self.assertEqual(params['jenkins_build'], obj['jenkins_build'])
        self.assertEqual(params['image'], obj['image'])

        # make sure one SmokeTestResult from above was created
        obj = SmokeResult.objects.get(pk=obj['id'])
        self.assertEqual(1, len(obj.smoketestresult_set.values()))

    def _testResultState(self, state):
        resource = '/smokeng/api/v1/result/%d/' % self.result.id
        params = {
            'ran_at': datetime.datetime.now().isoformat(),
            'status': state,
            'total_count': 6,
            'pass_count': 3,
            'error_count': 2,
            'fail_count': 1,
        }
        self._patch(resource, params)
        res = SmokeResult.objects.get(pk=self.result.id)
        self.assertEqual(self.result.name, res.name)
        self.assertEqual(state, res.status)

    def testResultStates(self):
        self._testResultState(SmokeResult.QUEUED)
        self._testResultState(SmokeResult.RUNNING)
        self._testResultState(SmokeResult.SYNCING)
        self._testResultState(SmokeResult.COMPLETE)

    def _testPullInteraction(self, as_string):
        '''The ran_at field can be both strings and datetime objects'''
        ran_at = datetime.datetime.now()
        if as_string:
            ran_at = ran_at.strftime('%Y-%m-%d %H:%M:%S')

        data = {
            # total/passes/errors/failures come from _testResultState
            'test_name': self.result.name,
            'total': 6,
            'passes': 3,
            'errors': 2,
            'failures': 1,
            'ran_at': ran_at,
        }
        self._testResultState(SmokeResult.SYNCING)
        c = Command()
        c.image = self.image
        c.bugs = []
        c.install_data = {
            'test_name': self.result.name,
            'jenkins_build': self.jenkins_build
        }
        c.add_result(data)

        res = SmokeResult.objects.get(pk=self.result.id)
        self.assertEqual(self.result.name, res.name)
        self.assertEqual(SmokeResult.COMPLETE, res.status)

    def testPullAsString(self):
        self._testPullInteraction(True)

    def testPullAsDatetime(self):
        self._testPullInteraction(False)


class TestCategories(TestCase):

    """Test bug categorization functions."""

    def setUp(self):
        _setUp(self)
        self.url = reverse(
            'smokeng_test_result_detail',
            args=(
                self.image.release,
                self.image.variant,
                self.image.arch,
                self.image.build_number,
                self.image.id,
                self.result.name,
                self.testcaseresult.pk,
            ),
        )
        self.user = User.objects.create(
            username='testuser',
            first_name='Test',
            last_name='User',
        )
        self.user.set_password('password')
        self.user.save()
        self.assert_(self.user)
        self.group_name = 'image-test-triage'
        self.title = 'Failure explanation'
        self.edit = '>edit</a>)'
        self.not_categorized = 'This test failure has not yet been categorized'
        self.cause = 'Cause'
        self.categorize = 'to edit.'
        self.form_inc = '<form action="" method="POST">'

    def _login(self):
        logged_in = self.client.login(
            username='testuser',
            password='password',
        )
        self.assertTrue(logged_in)

    def _add_to_group(self):
        Group.objects.get(name=self.group_name).user_set.add(self.user)
        self.assertTrue(
            self.user.groups.filter(name=self.group_name).exists(),
            'User not in triage group',
        )

    def test_default_value(self):
        self.assertIsNone(self.image.categories())
        self.assertIsNone(self.result.categories())
        self.assertEqual(self.testcaseresult.category, '')

    def test_aggregation(self):
        expectedResult = {
            'Application Regression': '1 (20%)',
            'Infrastructure Issue': '1 (20%)',
            'Test Problem': '1 (20%)',
            'Autopilot Bug': '1 (20%)',
            'Uncategorized': '1 (20%)',
        }
        for category, _name in SmokeTestResult.CATEGORY_CHOICES:
                self.testcaseresult = SmokeTestResult.objects.create(
                    result=self.result,
                    command_type="testcase_test",
                    command="./test.py",
                    testcase="test-testcase",
                    testsuite="touch",
                    returncode="1",
                    category=category,
                )
        self.assertEqual(self.image.categories(), expectedResult)
        self.assertEqual(self.result.categories(), expectedResult)

    def test_no_category(self):
        """
        Test that a result with Null category doesn't show the box
        """
        self.client.logout()
        self.testcaseresult.category = ''
        self.testcaseresult.save()
        resp = self.client.get(self.url)
        self.assertNotIn(
            self.title,
            resp.content,
        )
        self.assertNotIn(
            self.not_categorized,
            resp.content,
        )
        self.assertNotIn(
            self.cause,
            resp.content,
        )
        self.assertNotIn(
            self.categorize,
            resp.content,
        )
        self.assertNotIn(
            self.form_inc,
            resp.content,
        )
        self.assertNotIn(
            self.edit,
            resp.content,
        )

    def test_user_not_logged_in(self):
        """
        Test that when a user is not logged in and the failure is not
        categorized they are given a link to login
        """
        self.client.logout()
        self.testcaseresult.category = 'U'
        self.testcaseresult.save()
        resp = self.client.get(self.url)
        self.assertIn(
            self.title,
            resp.content,
        )
        self.assertIn(
            self.not_categorized,
            resp.content,
        )
        self.assertNotIn(
            self.cause,
            resp.content,
        )
        self.assertIn(
            self.categorize,
            resp.content,
        )
        self.assertNotIn(
            self.form_inc,
            resp.content,
        )
        self.assertNotIn(
            self.edit,
            resp.content,
        )

    def test_user_not_logged_in_already_categorized(self):
        """
        Test that when a user is not logged in and the failure is categorized
        they are shown the categorization
        """
        self.client.logout()
        self.testcaseresult.category = 'A'
        self.testcaseresult.save()
        resp = self.client.get(self.url)
        self.assertIn(
            self.title,
            resp.content,
        )
        self.assertNotIn(
            self.not_categorized,
            resp.content,
        )
        self.assertIn(
            self.cause,
            resp.content,
        )
        self.assertNotIn(
            self.categorize,
            resp.content,
        )
        self.assertNotIn(
            self.form_inc,
            resp.content,
        )
        self.assertNotIn(
            self.edit,
            resp.content,
        )

    def test_user_logged_in_not_in_team_not_categorized(self):
        """
        Test that a user who is logged in but isn't in an appropriate team is
        shown that the failure has not been categorized, and is not given
        the ability to edit the failure
        """
        self._login()
        self.testcaseresult.category = 'U'
        self.testcaseresult.save()
        resp = self.client.get(self.url)
        self.assertIn(
            self.title,
            resp.content,
        )
        self.assertIn(
            self.not_categorized,
            resp.content,
        )
        self.assertNotIn(
            self.cause,
            resp.content,
        )
        self.assertNotIn(
            self.categorize,
            resp.content,
        )
        self.assertNotIn(
            self.form_inc,
            resp.content,
        )
        self.assertNotIn(
            self.edit,
            resp.content,
        )

    def test_user_logged_in_not_in_team_categorized(self):
        """
        Test that a user who is logged in but isn't in an appropriate team is
        shown the failure categorization, but not the ability to edit the
        failure
        """
        self._login()
        self.testcaseresult.category = 'A'
        self.testcaseresult.save()
        resp = self.client.get(self.url)
        self.assertIn(
            self.title,
            resp.content,
        )
        self.assertNotIn(
            self.not_categorized,
            resp.content,
        )
        self.assertIn(
            self.cause,
            resp.content,
        )
        self.assertNotIn(
            self.categorize,
            resp.content,
        )
        self.assertNotIn(
            self.form_inc,
            resp.content,
        )
        self.assertNotIn(
            self.edit,
            resp.content,
        )

    def test_user_logged_in_with_permissions_not_categorized(self):
        """
        Test that a user who is logged in and is a member of the correct team
        is shown the form to edit the failure categorization
        """
        self._login()
        self._add_to_group()
        self.testcaseresult.category = 'U'
        self.testcaseresult.save()
        resp = self.client.get(self.url)
        self.assertIn(
            self.title,
            resp.content,
        )
        self.assertIn(
            self.not_categorized,
            resp.content,
        )
        self.assertIn(
            self.cause,
            resp.content,
        )
        self.assertNotIn(
            self.categorize,
            resp.content,
        )
        self.assertIn(
            self.form_inc,
            resp.content,
        )
        self.assertNotIn(
            self.edit,
            resp.content,
        )

    def test_user_logged_in_with_permissions_categorized(self):
        """
        Test that a user who is logged in and is a member of the correct team
        is not shown the form to edit the failure categorization
        """
        self._login()
        self._add_to_group()
        self.testcaseresult.category = 'A'
        self.testcaseresult.save()
        resp = self.client.get(self.url)
        self.assertIn(
            self.title,
            resp.content,
        )
        self.assertNotIn(
            self.not_categorized,
            resp.content,
        )
        self.assertIn(
            self.cause,
            resp.content,
        )
        self.assertNotIn(
            self.categorize,
            resp.content,
        )
        self.assertNotIn(
            self.form_inc,
            resp.content,
        )
        self.assertIn(
            self.edit,
            resp.content,
        )

    def test_user_logged_in_with_permissions_not_categorized_show_form(self):
        """
        Test that a user who is logged in and is a member of the correct team
        is shown the form to edit the failure categorization when asked
        """
        self._login()
        self._add_to_group()
        self.testcaseresult.category = 'A'
        self.testcaseresult.save()
        resp = self.client.get(self.url + '?form=True')
        self.assertIn(
            self.title,
            resp.content,
        )
        self.assertNotIn(
            self.not_categorized,
            resp.content,
        )
        self.assertIn(
            self.cause,
            resp.content,
        )
        self.assertNotIn(
            self.categorize,
            resp.content,
        )
        self.assertIn(
            self.form_inc,
            resp.content,
        )
        self.assertNotIn(
            self.edit,
            resp.content,
        )

    def test_form(self):
        """
        Test that an authorized user can save failure categorization form
        """
        self._login()
        self._add_to_group()
        resp = self.client.post(self.url, {'category': 'A'})
        tcr = SmokeTestResult.objects.get(id=self.testcaseresult.id)
        self.assertEqual(resp.status_code, 302, 'POST of form does not work')
        self.assertEqual(tcr.category, 'A', 'Category not set by form')
