# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
UTAH result parsing code.
"""

import logging

from common.utah_parser import UTAHParser, ParserError

from smokeng.models import (
    SmokeTestResult,
)


def process_smokeng_log(
    logfile,
    result,
    jenkins_url=None,
):
    """
    Parse a utah client log for smoke test results.
    """
    details = []

    parser = UTAHParser()

    try:
        data = parser.parse(jenkins_url)
    except ParserError as e:
        if not e.zero_entry:
            logging.error("%s %s", jenkins_url, e)
            return details

    if data is None:
        logging.warn("Unable to parse %s", jenkins_url)
        return details

    data['result'] = result

    _add_test_results(details, data)


def _add_test_results(details, data):
    for cmd in data['commands']:
        command_type = cmd['cmd_type']
        command = cmd['command']
        stderr = cmd['stderr']
        stdout = cmd['stdout']
        """
        In the past, Utah would return 'null' as a returncode in certain
        instances. The null return has now changed to return '15' when the
        issue happens in a test. Therefore we will set a returncode of 'null'
        to 15.
        Relevant Utah code: http://goo.gl/4CE6g
        """
        if isinstance(cmd['returncode'], int):
            returncode = cmd['returncode']
        elif cmd['returncode'] == 'null':
            returncode = 15

        extra_filters = {}
        if not command_type.startswith('testsuite_'):
            testcase = cmd['testcase']
            testsuite = cmd['testsuite']

            extra_filters = dict(
                testcase=testcase,
                testsuite=testsuite,
            )

        # clean up duplicates if we find any
        current = data['result'].smoketestresult_set.filter(
            publish=True,
            command_type=command_type,
            command=command,
            **extra_filters
        )

        if current.count() > 1:
            current.update(publish=False)

        testresult, new_testresult = SmokeTestResult.objects.get_or_create(
            publish=True,
            result=data['result'],
            command_type=command_type,
            command=command,
            defaults=dict(
                returncode=returncode,
                stderr=stderr,
                stdout=stdout,
            ),
            **extra_filters
        )

        if not new_testresult:
            testresult.returncode = returncode
            testresult.stderr = stderr
            testresult.stdout = stdout
            testresult.save()

        if testresult.returncode != 0:
            testresult.category = SmokeTestResult.UNCATEGORIZED
            testresult.save()

        details.append(testresult)

    return details
