# QA Dashboard
# Copyright 2012-2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import logging

from django.db import models
from django.shortcuts import (
    render_to_response,
    get_object_or_404,
    HttpResponseRedirect,
)
from django_tables2 import RequestConfig
from django.template import RequestContext
from django.views.decorators.http import (
    require_GET,
    require_http_methods,
)

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)

from common.models import Artifact
from common.views import index
from common.templatetags.dashboard_extras import decimal_to_percent

from smokeng.config import get_internal_jenkins, primary_targets
from smokeng.models import (
    SmokeImage,
    SmokeResult,
    SmokeResultBug,
    SmokeTestResult,
)
from smokeng.tables import (
    SmokeImageTable,
    SmokeResultTable,
    SmokeTestResultTable,
)
from smokeng.forms import EditCategoriesForm


def _get_jenkins_url(variant, user=None):
    if user.userprofile.use_private_url:
        return get_internal_jenkins(variant)


def _get_job_url(url):
    return re.sub(
        ur'^http(s|):\/\/jenkins.qa.ubuntu.com\/',
        '',
        url,
    )


def _get_releases():
    return [
        image['release'] for image in SmokeImage.objects.values(
            'release',
        ).order_by('-release').distinct().exclude(publish=False)
    ]


def _get_variants(release):
    return [
        image['variant'] for image in SmokeImage.objects.filter(
            release=release,
        ).values(
            'variant',
        ).order_by('-variant').distinct().exclude(publish=False)
    ]


def get_totals(results, release):
    totals = {}
    passrates = {}

    bugs = _get_bugs(release)
    crashes = _get_crashes()
    bug_data = {}
    for result in results:
        _append_totals(totals, result, crashes)
        if result['total_count'] == 0:
            passrate = 0
        else:
            passrate = float(result['pass_count']) / result['total_count']

        passrates.setdefault(result['image__id'], []).append(passrate)

    for image, passrate in passrates.iteritems():
        totals[image]['pass_rate'] = \
            sum(passrates[image]) / len(passrates[image])
        totals[image]['pass_rate_pct'] = \
            totals[image]['pass_rate'] * 100

    for bug in bugs.iterator():
        if bug.result.image.id in totals:
            if bug.bug.bug_no not in totals[bug.result.image.id]['bugs']:
                totals[bug.result.image.id]['bugs'].append(bug.bug.bug_no)
        if bug.bug.bug_no not in bug_data:
            bug_info = dict(
                bug_number=bug.bug.bug_no,
                title=bug.bug.title,
                assignee=bug.bug.assignee,
                status=bug.bug.status,
                importance=bug.bug.importance,
                project=bug.bug.project,
            )
            bug_data[bug.bug.bug_no] = bug_info
    return totals, bug_data


def _get_crashes():
    crashes = SmokeResult.objects.filter(
        publish=True,
        jenkins_build__artifact__name__endswith='.crash',
    ).distinct(
        'image__id',
        'jenkins_build__artifact__url',
    ).values(
        'image__id',
        'jenkins_build__id',
        'jenkins_build__artifact__url',
    )

    multi = re.compile('.*clientlogs/(.*)/.*')
    for c in crashes:
        m = multi.match(c['jenkins_build__artifact__url'])
        if m:
            c['smokeresult'] = m.group(1)
        else:
            c['smokeresult'] = ''
    return crashes


def _get_bugs(release):

    bugs = SmokeResultBug.objects.filter(
        result__image__release=release,
    ).select_related().order_by('bug')
    return bugs


def _append_totals(totals, result, crashes):
    key = result['image__id']

    if key not in totals:
        # we could have job-wide crashes or per-mega test crashes. figure
        # out job wide crashes here, and per-mega get appended below:
        job_crashes = len([x for x in crashes
                          if x['image__id'] == key and not x['smokeresult']])
        totals[key] = dict(
            build_id=result['image__build_id'],
            rootfs_id=result['image__rootfs_id'],
            device_specific_id=result['image__device_specific_id'],
            customization_id=result['image__customization_id'],
            build_number=result['image__build_number'],
            release=result['image__release'],
            total_count=0,
            pass_count=0,
            fail_count=0,
            error_count=0,
            bugs=[],
            name=result['name'],
            crash_count=job_crashes,
        )

        if 'image__variant' in result:
            totals[key]['variant'] = result['image__variant']

        if 'image__id' in result:
            totals[key]['image_id'] = result['image__id']

        if 'image__arch' in result:
            totals[key]['arch'] = result['image__arch']

        if 'ran_at' in result:
            totals[key]['ran_at'] = result['ran_at']

    if result['status'] < SmokeResult.SYNCING:
        totals[key]['inprogress'] = True

    totals[key]['total_count'] += result['total_count']
    totals[key]['pass_count'] += result['pass_count']
    totals[key]['error_count'] += result['error_count']
    totals[key]['fail_count'] += result['fail_count']

    for c in crashes:
        if c['jenkins_build__id'] == result['jenkins_build']:
            if result['name'] == c['smokeresult']:
                totals[key]['crash_count'] += 1


def _results_overview(results):
    return results.values(
        'image__build_number',
        'image__variant',
        'image__arch',
        'image__id',
        'image__build_id',
        'image__rootfs_id',
        'image__device_specific_id',
        'image__customization_id',
        'image__release',
        'name',
        'ran_at',
        'status',
        'jenkins_build',
    ).annotate(
        pass_count=models.Sum('pass_count'),
        fail_count=models.Sum('fail_count'),
        error_count=models.Sum('error_count'),
        total_count=models.Sum('total_count'),
    ).values(
        'image__build_id',
        'image__rootfs_id',
        'image__device_specific_id',
        'image__customization_id',
        'image__build_number',
        'image__variant',
        'image__arch',
        'image__id',
        'image__release',
        'name',
        'ran_at',
        'status',
        'jenkins_build',
        'pass_count',
        'fail_count',
        'error_count',
        'total_count',
    )


@BreadCrumb("{release} image testing", parent=index, needs=['release'])
@require_GET
def overview(request, release=None, variant=None):

    if release is None:
        releases = _get_releases()
        release = releases[0]

    variants = _get_variants(release)

    results = SmokeResult.objects.filter(
        publish=True,
        image__release=release,
    )
    targets = ''
    if variant is not None:
        results = results.filter(
            image__variant=variant,
        )

        primary_tgts = primary_targets(variant)
        if request.GET.get('show_all') != '1' and primary_tgts:
            targets = 'primary'
            results = results.filter(image__arch__in=primary_tgts)

    results = _results_overview(results)

    totals, bug_data = get_totals(results, release)
    table = SmokeImageTable(totals.values())
    RequestConfig(request, paginate=None).configure(table)

    data = {
        'table': table,
        'bug_data': bug_data,
        'release': release,
        'releases': _get_releases(),
        'variants': variants,
        'current': variant,
        'targets': targets,
        'active': release,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            overview,
            release=release,
        )
    }

    return render_to_response(
        'smokeng/overview.html',
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Image {build_number}",
    parent=overview,
    needs=['release', 'variant', 'arch', 'build_number', 'image_id'],
)
@require_GET
def image_overview(request, release, variant, arch, build_number, image_id):

    image = get_object_or_404(SmokeImage, id=image_id)
    summary, bug_data = image.summary

    table = SmokeResultTable(summary['results'])
    RequestConfig(request, paginate=None).configure(table)

    data = {
        'image': image,
        'bug_data': bug_data,
        'summary': summary,
        'build_number': image.build_number,
        'release': release,
        'categories': image.categories(),
        'table': table,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_overview,
            release=release,
            variant=variant,
            arch=arch,
            image_id=image_id,
            build_number=image.build_number,
        ),
    }
    return render_to_response(
        'smokeng/image_overview.html',
        data,
        RequestContext(request),
    )


def _test_history(smokeresult, limit=5):
    history = SmokeResult.objects.filter(
        name=smokeresult.name,
        image__release=smokeresult.image.release,
        image__variant=smokeresult.image.variant,
        image__arch=smokeresult.image.arch,
        image__rootfs_id__lt=smokeresult.image.rootfs_id,
        publish=True,
    ).order_by(
        '-image__rootfs_id'
    ).values(
        'pass_rate', 'image__build_number', 'image__id')[:limit]
    return history


def _result_artifacts(result):
    artifacts = Artifact.objects.filter(
        jenkins_build=result.jenkins_build,
    ).order_by('name')

    result_artifacts = []
    multi = re.compile('.*clientlogs/(.*)/.*')
    for a in artifacts:
        # ensure if its a mega-job its the proper result
        m = multi.match(a.url)
        if not m or m.group(1) == result.name:
            result_artifacts.append(a)
    return result_artifacts


@BreadCrumb(
    "{name}",
    parent=image_overview,
    needs=['release', 'variant', 'arch', 'build_number', 'image_id', 'name'],
)
@require_GET
def image_test_results(
    request,
    release,
    variant,
    arch,
    build_number,
    image_id,
    name,
):
    image = get_object_or_404(SmokeImage, id=image_id)
    result = get_object_or_404(
        SmokeResult,
        image__id=image_id,
        name=name,
        publish=True,
    )
    tests = SmokeTestResult.objects.filter(
        publish=True,
        result=result,
    )

    artifacts = _result_artifacts(result)

    private_url = None
    if hasattr(request.user, 'userprofile'):
        private_url = _get_jenkins_url(variant, request.user)

    if private_url:
        job_url = _get_job_url(result.jenkins_build.job.url)
        jenkins_url = (
            private_url +
            job_url +
            result.jenkins_build.build_number,
        )
        jenkins_url = jenkins_url[0]
    else:
        jenkins_url = (
            result.jenkins_build.job.url +
            result.jenkins_build.build_number
        )

    totals = {}
    totals['pass'] = result.pass_count
    totals['error'] = result.error_count
    totals['fail'] = result.fail_count
    totals['count'] = result.total_count
    totals['rate'] = decimal_to_percent(result.pass_rate)

    table_data = []
    for test in tests:
        test_data = dict(
            release=release,
            name=name,
            arch=arch,
            variant=variant,
            build_number=build_number,
            testcase=test.testcase,
            url_name=os.path.basename(test.testcase),
            testsuite=os.path.basename(test.testsuite),
            command_type=test.command_type,
            returncode=test.returncode,
            command=test.command,
            image_id=image.id,
        )
        test_data['id'] = test.pk
        table_data.append(test_data)

    table = SmokeTestResultTable(table_data)
    RequestConfig(request, paginate=None).configure(table)

    data = {
        'name': name,
        'image': image,
        'build_number': image.build_number,
        'release': release,
        'tests': tests,
        'totals': totals,
        'jenkins_url': jenkins_url,
        'private_url': private_url,
        'categories': result.categories(),
        'table': table,
        'artifacts': artifacts,
        'history': _test_history(result),
        'half': len(artifacts) / 2,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            image_test_results,
            release=release,
            variant=variant,
            arch=arch,
            image_id=image_id,
            name=name,
            build_number=image.build_number,
        ),
    }
    return render_to_response(
        'smokeng/test_results.html',
        data,
        RequestContext(request),
    )


def _result_history(testresult, limit=5):
    history = SmokeTestResult.objects.filter(
        testcase=testresult.testcase,
        result__name=testresult.result.name,
        result__image__release=testresult.result.image.release,
        result__image__variant=testresult.result.image.variant,
        result__image__arch=testresult.result.image.arch,
        result__image__rootfs_id__lt=testresult.result.image.rootfs_id,
        result__image__publish=True,
        result__publish=True,
        publish=True,
    ).order_by(
        '-result__image__rootfs_id'
    ).values(
        'returncode', 'result__image__build_number', 'result__image__id',
        'id')[:limit]
    return history


@BreadCrumb(
    "Testcase details",
    parent=image_test_results,
    needs=[
        'release',
        'variant',
        'arch',
        'build_number',
        'image_id',
        'name',
        'id',
    ],
)
@require_http_methods(['GET', 'POST'])
def test_result_detail(
    request,
    release,
    variant,
    arch,
    build_number,
    image_id,
    name,
    id,
):
    test = get_object_or_404(
        SmokeTestResult,
        pk=id,
    )
    artifacts = _result_artifacts(test.result)

    can_triage = request.user.groups.filter(name='image-test-triage').exists()

    if request.method == 'POST':
        if can_triage:
            logging.debug('Processing category form')
            form = EditCategoriesForm(data=request.POST, instance=test)
            if form.is_valid():
                form.save()
                return HttpResponseRedirect(request.path)
        else:
            logging.error('POST received from unauthorized user')
            form = EditCategoriesForm(initial={'category': test.category})
    else:
        form = EditCategoriesForm(initial={'category': test.category})

    show_form = can_triage and (
        test.category == SmokeTestResult.UNCATEGORIZED or
        'form' in request.GET)

    results = []
    test_data = dict(
        command_type=test.command_type,
        testsuite=test.testsuite,
        testcase=test.testcase,
        command=test.command,
        returncode=test.returncode,
        stderr=test.stderr,
        stdout=test.stdout,
    )
    results.append(test_data)

    private_url = None

    if hasattr(request.user, 'userprofile'):
        private_url = _get_jenkins_url(variant, request.user)

    if private_url:
        job_url = _get_job_url(test.result.jenkins_build.job.url)
        jenkins_url = (
            private_url +
            job_url +
            test.result.jenkins_build.build_number,
        )
        jenkins_url = jenkins_url[0]
    else:
        jenkins_url = (
            test.result.jenkins_build.job.url +
            test.result.jenkins_build.build_number
        )

    data = {
        'results': results,
        'name': name,
        'image': test.result.image,
        'build_number': test.result.image.build_number,
        'release': test.result.image.release,
        'category': test.get_category_display(),
        'jenkins_url': jenkins_url,
        'private_url': private_url,
        'can_triage': can_triage,
        'show_form': show_form,
        'testcase': test.testcase,
        'artifacts': artifacts,
        'history': _result_history(test),
        'half': len(artifacts) / 2,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            test_result_detail,
            release=release,
            variant=variant,
            arch=arch,
            image_id=image_id,
            name=name,
            id=id,
            build_number=test.result.image.build_number,
        ),
        'formset': form,
    }

    return render_to_response(
        'smokeng/test_result_detail.html',
        data,
        RequestContext(request),
    )
