# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from datetime import datetime

from common.management import (
    JenkinsBaseCommand,
)
from common.utils import regexes
from sru.models import Kernel, SRUResult


def _get_kernel_version(description, default="Unknown"):
    """
    Parse a jenkins build description for a kernel version
    """
    version = default

    if description is None:
        description = ''

    r = regexes['kernel_sru-version']

    m = r.match(description)

    if m:
        version = m.group(1)

    return version


class Command(JenkinsBaseCommand):
    job_regex = regexes['kernel_sru']

    def extract_data(self, name):
        m = self.job_regex.match(name)
        if m:
            pre_job_type = m.group(1)
            post_job_type = m.group(2)
            self.job_type = "{}{}".format(pre_job_type, post_job_type)
            pre_release = m.group(3)
            post_release = m.group(4)
            self.release = "{}{}".format(pre_release, post_release)
            self.variant = m.group(5)
            self.arch = m.group(6)
            self.gpu = m.group(7)

            self.install_data = dict(
                job_type=self.job_type,
                release=self.release,
                variant=self.variant,
                arch=self.arch,
                gpu=self.gpu,
            )

    def process_job(self, job):
        pass

    def _process_logs(self, build, build_date, install_data, func):
        if not hasattr(func, '__call__'):
            raise Exception("invalid callable: %s", func)
        build_name = build['fullDisplayName'].split(' ')[0]
        build_number = build['number']
        build_url = build['url']
        build_desc = build['description']
        build_date = datetime.fromtimestamp(
            build['timestamp']/1000
        )
        actions = build['actions']

        if build_desc is None:
            build_desc = ''

        kernel_version = _get_kernel_version(
            build_desc,
            "Unknown. Ran on {}".format(
                build_date.strftime("%Y-%m-%d %H:%M")
            )
        )
        dashboard_data = dict(
            build_name=build_name,
            build_date=build_date,
            build_number=build_number,
            build_desc=build_desc,
            build_url=build_url,
            actions=actions,
            variant=install_data['variant'],
            arch=install_data['arch'],
            kernel=kernel_version,
            job_type=install_data['job_type'],
            release=install_data['release'],
            gpu=install_data['gpu'],
        )

        func(build, dashboard_data)

    def add_result(self, build, dashboard_data):
        kernel, new_kernel = Kernel.objects.get_or_create(
            release=dashboard_data['release'],
            variant=dashboard_data['variant'],
            arch=dashboard_data['arch'],
            gpu=dashboard_data['gpu'],
            version=dashboard_data['kernel'],
            job_type=dashboard_data['job_type'],
        )

        fail_count = 0
        skip_count = 0
        pass_count = 0
        total_count = 0

        for action in dashboard_data['actions']:
            if action is None:
                logging.warn(
                    "missing actions for build: %s",
                    build['url'],
                )
                continue
            try:
                fail_count = action['failCount']
                skip_count = action['skipCount']
                total_count = action['totalCount']
                pass_count = total_count - fail_count - skip_count
                break
            except KeyError:
                continue

        old_results = kernel.sruresult_set.filter(
            name=dashboard_data['build_name'],
        ).update(publish=False)

        if old_results:
            logging.info("Setting results to publish=False: %s", old_results)

        result, new_result = kernel.sruresult_set.get_or_create(
            name=dashboard_data['build_name'],
            jenkins_build=dashboard_data['build_number'],
            defaults={
                'ran_at': dashboard_data['build_date'],
                'jenkins_url': dashboard_data['build_url'],
                'fail_count': fail_count,
                'skip_count': skip_count,
                'pass_count': pass_count,
                'total_count': total_count,
            }
        )

        if not new_result:
            result.publish = True
            result.save()

    def remove_result(self, build, dashboard_data):
        """ Remove sru data from the database """
        logging.info("Removing sru result")

        sru = SRUResult.objects.filter(
            name=dashboard_data['build_name'],
            jenkins_build=dashboard_data['build_number'],
            ran_at=dashboard_data['build_date'],
            jenkins_url=dashboard_data['build_url'],
        )
        logging.info("Removing %s", sru)
        sru.delete()
