# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models

from common.models import DashboardBaseModel, Bug


class Kernel(DashboardBaseModel):
    class Meta:
        db_table = "kernels"

    job_type = models.CharField(max_length=4096)
    release = models.CharField(max_length=4096)
    variant = models.CharField(max_length=4096)
    arch = models.CharField(max_length=4096)
    gpu = models.CharField(max_length=4096)
    version = models.CharField(max_length=4096)

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
        'pass_count',
        'skip_count',
        'fail_count',
        'total_count',
        'pass_rate',
        'pass_rate_pct',
        'bug_count',
        'last_run_date',
    ]

    def __unicode__(self):
        return self.debug_str()

    def debug_str(self):
        return "{}-{}-{}-{}".format(
            self.release,
            self.variant,
            self.arch,
            self.gpu,
        )

    @property
    def name(self):
        return "{}-{}".format(self.release, self.variant)

    @property
    def total_count(self):
        return self.sruresult_set.exclude(
            publish=False,
        ).aggregate(
            models.Sum('total_count')
        )['total_count__sum'] or 0

    @property
    def pass_count(self):
        return self.sruresult_set.exclude(
            publish=False,
        ).aggregate(
            models.Sum('pass_count')
        )['pass_count__sum'] or 0

    @property
    def fail_count(self):
        return self.sruresult_set.exclude(
            publish=False,
        ).aggregate(
            models.Sum('fail_count')
        )['fail_count__sum'] or 0

    @property
    def skip_count(self):
        return self.sruresult_set.exclude(
            publish=False,
        ).aggregate(
            models.Sum('skip_count')
        )['skip_count__sum'] or 0

    @property
    def pass_rate(self):
        if self.total_count > 0:
            return self.pass_count * 1.0 / self.total_count
        else:
            return 0

    @property
    def pass_rate_pct(self):
        return "%d%%" % int(self.pass_rate*100)

    @property
    def bug_count(self):
        return self.sruresult_set.exclude(
            publish=False,
        ).annotate(
            num_bugs=models.Count('bugs')
        ).aggregate(models.Sum('num_bugs'))['num_bugs__sum'] or 0

    @property
    def last_run_date(self):
        return self.sruresult_set.exclude(
            publish=False,
        ).aggregate(
            models.Max('ran_at')
        )['ran_at__max'].strftime("%Y-%m-%d")


# XXX: refactor this and Result
class SRUResult(DashboardBaseModel):
    class Meta:
        db_table = 'sru_results'

    kernel = models.ForeignKey(Kernel)
    bugs = models.ManyToManyField(Bug)

    jenkins_build = models.CharField(max_length=4096)
    jenkins_url = models.URLField(max_length=4096)
    name = models.CharField(max_length=4096)
    pass_count = models.IntegerField()
    fail_count = models.IntegerField()
    skip_count = models.IntegerField()
    total_count = models.IntegerField()
    ran_at = models.DateTimeField('date run')

    def __unicode__(self):
        return "{} - {}".format(
            self.name,
            self.ran_at,
        )

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
        'pass_rate',
        'pass_rate_pct',
        'bug_count',
    ]

    @property
    def pass_rate(self):
        if self.total_count > 0:
            return self.pass_count * 1.0 / self.total_count
        else:
            return 0

    @property
    def pass_rate_pct(self):
        return "%d%%" % int(self.pass_rate*100)

    @property
    def bug_count(self):
        return self.bugs.count()
