# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db.models import Sum, Count, Max
from django.shortcuts import render_to_response
from django.views.decorators.http import require_GET
from django.template import RequestContext
from datetime import date, timedelta
from django_tables2 import RequestConfig

from common.bread_crumbs import (
    BreadCrumb,
    BreadCrumbTrail,
)

from common.views import index
from sru.models import Kernel, SRUResult
from sru.tables import SRUKernelTable, SRUFlatKernelTable, SRUResultTable


def _get_kernel_names():
    kernels = []
    for kernel in Kernel.objects.values('variant').distinct().exclude(
        publish=False
    ).order_by('variant'):
        kernels.append(kernel['variant'])

    return kernels


def _get_kernel_releases():
    releases = []
    for kernel in Kernel.objects.values('release').distinct().exclude(
        publish=False
    ).order_by('-release'):
        releases.append(kernel['release'])

    return releases


@BreadCrumb("SRU Testing", parent=index)
@require_GET
def overview(request, release='', flat=False, all=False):
    past_date = date.today() - timedelta(days=30)

    data = {
        'releases': _get_kernel_releases(),
        'active': release,
        'show_unknown': True,
        'show_all': True,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(overview),
    }

    results = SRUResult.objects.select_related(
        'kernel',
    ).exclude(
        kernel__publish=False,
    ).exclude(
        publish=False,
    )

    if not all:
        data['show_all'] = False
        results = results.filter(ran_at__gte=past_date)

    if release != '':
        data['release'] = release
        results = results.filter(kernel__release=release)

    show_unknown = request.GET.get('show_unknown')
    if show_unknown is None or show_unknown != "1":
        data['show_unknown'] = False
        results = results.exclude(
            kernel__version__startswith="Unknown",
        )

    results = results.values(
        'kernel__version',
    ).annotate(
        bug_count=Count('bugs'),
        pass_count=Sum('pass_count'),
        fail_count=Sum('fail_count'),
        total_count=Sum('total_count'),
        last_ran_at=Max('ran_at'),
    )

    if flat:
        data['is_flat'] = True
        results = results.values(
            'kernel__version',
            'kernel__arch',
            'kernel__release',
            'pass_count',
            'fail_count',
            'total_count',
            'bug_count',
            'last_ran_at',
        )
        return _process_overview(
            request,
            results,
            data,
        )
    else:
        data['is_flat'] = False
        results = results.values(
            'kernel__version',
            'kernel__release',
            'pass_count',
            'fail_count',
            'total_count',
            'bug_count',
            'last_ran_at',
        )
        return _process_overview(
            request,
            results,
            data,
        )


def _process_overview(request, results, data):

    table_data = []
    for result in results:
        result['last_ran_at'] = result['last_ran_at'].strftime("%Y-%m-%d")

        if result['total_count'] > 0:
            pass_rate = result['pass_count'] * 1.0 / result['total_count']
        else:
            pass_rate = 0

        if data['is_flat']:
            arch = result['kernel__arch']
        else:
            arch = ''

        table_data.append(
            dict(
                arch=arch,
                release=result['kernel__release'],
                version=result['kernel__version'],
                last_run_date=result['last_ran_at'],
                total_count=result['total_count'],
                pass_count=result['pass_count'],
                fail_count=result['fail_count'],
                pass_rate=pass_rate,
                bug_count=result['bug_count'],
            )
        )

    if data['is_flat']:
        table = SRUFlatKernelTable(table_data)
    else:
        table = SRUKernelTable(table_data)

    if data['show_all']:
        RequestConfig(request, paginate={"per_page": 100}).configure(table)
    else:
        RequestConfig(request, paginate=None).configure(table)

    data['table'] = table
    return render_to_response(
        'sru/overview.html',
        data,
        RequestContext(request)
    )


@BreadCrumb(
    "Summary of {kernel}",
    parent=overview,
    needs=['release', 'kernel'],
)
@require_GET
def overview_kernel(request, release=None, kernel_version=None):
    results = SRUResult.objects.filter(
        kernel__version=kernel_version,
    ).select_related(
        'kernel',
    ).exclude(
        publish=False,
    ).values('kernel__version').annotate(
        bug_count=Count('bugs'),
        pass_total=Sum('pass_count'),
        fail_total=Sum('fail_count'),
        skip_total=Sum('skip_count'),
        total=Sum('total_count'),
    ).values(
        'kernel__variant',
        'kernel__arch',
        'kernel__gpu',
        'pass_total',
        'fail_total',
        'skip_total',
        'total',
        'bug_count',
        'ran_at',
        'jenkins_url',
    )

    totals = {
        'skip_count': 0,
        'pass_count': 0,
        'fail_count': 0,
        'total_count': 0,
        'pass_rate': 0.0,
    }
    table_data = []
    for result in results:
        if result['total'] > 0:
            pass_rate = result['pass_total'] * 1.0 / result['total']
        else:
            pass_rate = 0

        table_data.append(
            dict(
                arch=result['kernel__arch'],
                gpu=result['kernel__gpu'],
                variant=result['kernel__variant'],
                ran_at=result['ran_at'],
                total_count=result['total'],
                pass_count=result['pass_total'],
                fail_count=result['fail_total'],
                pass_rate=pass_rate,
                bug_count=result['bug_count'],
                jenkins_url=result['jenkins_url'],
            )
        )

        totals['skip_count'] += result['skip_total']
        totals['pass_count'] += result['pass_total']
        totals['fail_count'] += result['fail_total']
        totals['total_count'] += result['total']

    if totals['total_count'] == 0:
        totals['pass_rate'] = 0.0
    else:
        totals['pass_rate'] = totals['pass_count'] * 1.0
        totals['pass_rate'] = totals['pass_rate'] / totals['total_count']
    totals['pass_rate_pct'] = "{:.2%}".format(totals['pass_rate'])

    table = SRUResultTable(table_data)
    RequestConfig(request, paginate=None).configure(table)
    data = {
        'kernel_version': kernel_version,
        'release': release,
        'table': table,
        'totals': totals,
        'bread_crumb_trail': BreadCrumbTrail.leading_to(
            overview_kernel,
            release=release,
            kernel=kernel_version,
        ),
    }

    return render_to_response(
        'sru/overview_kernel.html',
        data,
        RequestContext(request)
    )
